package org.springblade.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springblade.core.tool.api.R;
import org.springblade.db.exception.DataException;
import org.springblade.model.dto.DataInterfaceCrForm;
import org.springblade.model.dto.PaginationDataInterface;
import org.springblade.model.po.DataInterfaceEntity;
import org.springblade.model.po.DictionaryDataEntity;
import org.springblade.model.vo.DataInterfaceListVO;
import org.springblade.model.vo.DataInterfaceTreeModel;
import org.springblade.model.vo.DataInterfaceTreeVO;
import org.springblade.model.vo.DataInterfaceVo;
import org.springblade.model.vo.ListVO;
import org.springblade.model.vo.PageListVO;
import org.springblade.model.vo.PaginationVO;
import org.springblade.model.vo.SumTree;
import org.springblade.model.vo.UserAllModel;
import org.springblade.service.DataInterfaceService;
import org.springblade.service.DictionaryDataService;
import org.springblade.service.UserService;
import org.springblade.util.TreeDotUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2021/3/12 15:31
 */
@Api(tags = "数据接口", value = "DataInterface")
@RestController
@RequestMapping(value = "/paas/system/DataInterface")
public class DataInterfaceController {
	@Autowired
	private DataInterfaceService dataInterfaceService;
	@Autowired
	private UserService userService;
	@Autowired
	private DictionaryDataService dictionaryDataService;

	/**
	 * 获取接口列表(分页)
	 *
	 * @param pagination
	 * @return
	 */
	@ApiOperation("获取接口列表(分页)")
	@GetMapping
	public R<PageListVO<DataInterfaceListVO>> getList(PaginationDataInterface pagination) {
		List<DataInterfaceEntity> data = dataInterfaceService.getList(pagination);
		List<UserAllModel> userServiceAll = userService.getAll();
		for (DataInterfaceEntity entity : data) {
			UserAllModel userAllVO = userServiceAll.stream()
					.filter(t -> t.getId().equals(entity.getCreatorUser()))
					.findFirst()
					.orElse(null);
			if (userAllVO != null) {
				entity.setCreatorUser(userAllVO.getRealName() + "/" + userAllVO.getAccount());
			}
		}
		List<DataInterfaceListVO> list = JSON.parseArray(JSON.toJSONString(data), DataInterfaceListVO.class);
		PaginationVO paginationVO = JSON.parseObject(JSON.toJSONString(pagination), PaginationVO.class);

		PageListVO<DataInterfaceListVO> pageData = new PageListVO<>();
		pageData.setList(list);
		pageData.setPagination(paginationVO);
		return R.data(pageData);
	}

	/**
	 * 获取接口列表下拉框
	 *
	 * @return
	 */
	@ApiOperation("获取接口列表下拉框")
	@GetMapping("/Selector")
	public R<List<DataInterfaceTreeVO>> getSelector() {
		List<DataInterfaceTreeModel> tree = new ArrayList<>();
		List<DataInterfaceEntity> data = dataInterfaceService.getList();
		for (DataInterfaceEntity entity : data) {
			DictionaryDataEntity dictionaryDataEntity = dictionaryDataService.getInfo(entity.getCategoryId());
			if (dictionaryDataEntity != null) {
				DataInterfaceTreeModel firstModel = JSON.parseObject(JSON.toJSONString(dictionaryDataEntity),
						DataInterfaceTreeModel.class);
				firstModel.setCategoryId("0");
				tree.add(firstModel);
				DataInterfaceTreeModel treeModel = JSON.parseObject(JSON.toJSONString(entity),
						DataInterfaceTreeModel.class);
				treeModel.setCategoryId("1");
				treeModel.setParentId(dictionaryDataEntity.getId());
				tree.add(treeModel);
			}
		}
		tree = tree.stream().distinct().collect(Collectors.toList());
		List<SumTree<DataInterfaceTreeModel>> sumTrees = TreeDotUtils.convertListToTreeDot(tree);
		List<DataInterfaceTreeVO> list = JSON.parseArray(JSON.toJSONString(sumTrees), DataInterfaceTreeVO.class);
		ListVO<DataInterfaceTreeVO> vo = new ListVO<>();
		vo.setList(list);
		return R.data(list);
	}

	/**
	 * 获取接口数据
	 *
	 * @param id
	 * @return
	 */
	@ApiOperation("获取接口数据")
	@GetMapping("/{id}")
	public R<DataInterfaceVo> getInfo(@PathVariable("id") String id) throws DataException {
		DataInterfaceEntity entity = dataInterfaceService.getById(id);
		DataInterfaceVo vo = JSON.parseObject(JSON.toJSONString(entity), DataInterfaceVo.class);
		if (vo.getCheckType() == null) {
			vo.setCheckType(0);
		}
		return R.data(vo);
	}

	/**
	 * 添加接口
	 *
	 * @param dataInterfaceCrForm
	 * @return
	 */
	@ApiOperation("添加接口")
	@PostMapping
	public R<String> create(@RequestBody @Valid DataInterfaceCrForm dataInterfaceCrForm) {
		DataInterfaceEntity entity = JSON.parseObject(JSON.toJSONString(dataInterfaceCrForm),
				DataInterfaceEntity.class);
		if (dataInterfaceService.isExistByFullName(entity.getFullName(), entity.getId())) {
			return R.fail("名称不能重复");
		}
		if (dataInterfaceService.isExistByEnCode(entity.getEnCode(), entity.getId())) {
			return R.fail("编码不能重复");
		}
		dataInterfaceService.save(entity);
		return R.success("接口创建成功");
	}

	/**
	 * 修改接口
	 *
	 * @param dataInterfaceUpForm
	 * @param id
	 * @return
	 */
	@ApiOperation("修改接口")
	@PutMapping("/{id}")
	public R<String> update(@RequestBody @Valid DataInterfaceCrForm dataInterfaceUpForm,
			@PathVariable("id") String id) throws DataException {
		DataInterfaceEntity entity = JSON.parseObject(JSON.toJSONString(dataInterfaceUpForm),
				DataInterfaceEntity.class);
		if (dataInterfaceService.isExistByFullName(entity.getFullName(), id)) {
			return R.fail("名称不能重复");
		}
		if (dataInterfaceService.isExistByEnCode(entity.getEnCode(), id)) {
			return R.fail("编码不能重复");
		}
		boolean flag = dataInterfaceService.updateById(entity);
		if (flag == false) {
			return R.fail("接口修改失败，数据不存在");
		}
		return R.success("接口修改成功");
	}

	/**
	 * 删除接口
	 *
	 * @param id
	 * @return
	 */
	@ApiOperation("删除接口")
	@DeleteMapping("/{id}")
	public R<String> delete(@PathVariable String id) {
		DataInterfaceEntity entity = dataInterfaceService.getById(id);
		if (entity != null) {
			dataInterfaceService.removeById(id);
			return R.success("删除成功");
		}
		return R.fail("删除失败，数据不存在");
	}

	/**
	 * 更新接口状态
	 *
	 * @param id
	 * @return
	 */
	@ApiOperation("更新接口状态")
	@PutMapping("/{id}/Actions/State")
	public R<String> update(@PathVariable("id") String id) throws DataException {
		DataInterfaceEntity entity = dataInterfaceService.getById(id);
		if (entity != null) {
			if ("0".equals(String.valueOf(entity.getEnabledMark()))) {
				entity.setEnabledMark(1);
			} else {
				entity.setEnabledMark(0);
			}
			dataInterfaceService.updateById(entity);
			return R.success("更新接口状态成功");
		}
		return R.fail("更新接口状态失败，数据不存在");
	}

}
