package org.springblade.controller;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.core.tool.api.R;
import org.springblade.db.exception.DataException;
import org.springblade.model.dto.Pagination;
import org.springblade.model.po.OrganizeEntity;
import org.springblade.model.po.UserEntity;
import org.springblade.model.vo.ListVO;
import org.springblade.model.vo.OrganizeDepartInfoVO;
import org.springblade.model.vo.OrganizeDepartListVO;
import org.springblade.model.vo.OrganizeDepartSelectorListVO;
import org.springblade.model.vo.OrganizeInfoVO;
import org.springblade.model.vo.OrganizeListVO;
import org.springblade.model.vo.OrganizeModel;
import org.springblade.model.vo.OrganizeSelectorVO;
import org.springblade.model.vo.OrganizeTreeVO;
import org.springblade.model.vo.SumTree;
import org.springblade.service.OrganizeService;
import org.springblade.service.UserService;
import org.springblade.util.ListToTreeUtil;
import org.springblade.util.TreeDotUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 数据处理
 *
 * @author william
 * @date 2022-07-14 10:57:52
 */
@Api(tags = "部门管理", value = "Organize")
@RequestMapping("/paas/Organize")
@RestController
@SuppressWarnings("unlikely-arg-type")
public class OrganizeController {
	@Autowired
	private OrganizeService organizeService;

	@Autowired
	private UserService userService;

	/**
	 * 获取组织列表
	 *
	 * @param pagination
	 * @return
	 */
	@ApiOperation("获取组织列表")
	@GetMapping
	public R<ListVO<OrganizeListVO>> getList(Pagination pagination) {
		List<OrganizeEntity> data = organizeService.getList();
		List<OrganizeEntity> dataAll = data;
		List<OrganizeEntity> list = JSON.parseArray(JSON.toJSONString(ListToTreeUtil.treeWhere(data, dataAll)),
				OrganizeEntity.class);
		list = list.stream().filter(t -> "company".equals(t.getCategory())).collect(Collectors.toList());
		List<OrganizeModel> oraganizeList = JSON.parseArray(JSON.toJSONString(list), OrganizeModel.class);
		List<SumTree<OrganizeModel>> trees = TreeDotUtils.convertListToTreeDot(oraganizeList);
		List<OrganizeListVO> listVO = JSON.parseArray(JSON.toJSONString(trees), OrganizeListVO.class);

		if (!StrUtil.isEmpty(pagination.getKeyword())) {
			listVO = listVO.stream()
					.filter(t -> t.getFullName().toLowerCase().contains(pagination.getKeyword())
							|| t.getEnCode().toLowerCase().contains(pagination.getKeyword()))
					.collect(Collectors.toList());
		}
		ListVO<OrganizeListVO> vo = new ListVO<>();
		vo.setList(listVO);
		return R.data(vo);
	}

	/**
	 * 获取组织下拉框列表
	 *
	 * @return
	 */
	@ApiOperation("获取组织下拉框列表")
	@GetMapping("/Selector")
	public R<ListVO<OrganizeSelectorVO>> getSelector(Pagination pagination) {
		List<OrganizeEntity> allList = organizeService.getList();
		List<OrganizeEntity> data = allList.stream()
				.filter(t -> "0".equals(String.valueOf(t.getIsDeleted())))
				.collect(Collectors.toList());
		List<OrganizeEntity> dataAll = data;
		if (!StrUtil.isEmpty(pagination.getKeyword())) {
			data = data.stream()
					.filter(t -> t.getFullName().contains(pagination.getKeyword())
							|| t.getEnCode().contains(pagination.getKeyword()))
					.collect(Collectors.toList());
		}
		List<OrganizeEntity> list = JSON.parseArray(JSON.toJSONString(ListToTreeUtil.treeWhere(data, dataAll)),
				OrganizeEntity.class);
		list = list.stream().filter(t -> "company".equals(t.getCategory())).collect(Collectors.toList());
		List<OrganizeModel> models = JSON.parseArray(JSON.toJSONString(list), OrganizeModel.class);
		for (OrganizeModel model : models) {
			model.setIcon("icon-ym icon-ym-tree-organization3");
		}
		List<OrganizeModel> modelAll = new ArrayList<>();
		modelAll.addAll(models);
		List<SumTree<OrganizeModel>> trees = TreeDotUtils.convertListToTreeDot(modelAll);
		List<OrganizeSelectorVO> listVO = JSON.parseArray(JSON.toJSONString(trees), OrganizeSelectorVO.class);

		// 将子节点全部删除
		Iterator<OrganizeSelectorVO> iterator = listVO.iterator();
		while (iterator.hasNext()) {
			OrganizeSelectorVO oraganizeSelectorVO = iterator.next();
			if (!"-1".equals(oraganizeSelectorVO.getParentId()) && !"0".equals(oraganizeSelectorVO.getParentId())) {
				iterator.remove();
			}
		}
		ListVO<OrganizeSelectorVO> vo = new ListVO<>();
		vo.setList(listVO);
		return R.data(vo);
	}

	/**
	 * 组织树形
	 *
	 * @return
	 */
	@ApiOperation("获取组织/公司树形")
	@GetMapping("/Tree")
	public R<ListVO<OrganizeTreeVO>> tree() {
		List<OrganizeEntity> allList = organizeService.getList();
		List<OrganizeEntity> list = allList.stream()
				.filter(t -> "0".equals(String.valueOf(t.getIsDeleted())))
				.collect(Collectors.toList());
		list = list.stream().filter(t -> "company".equals(t.getCategory())).collect(Collectors.toList());
		List<OrganizeModel> models = JSON.parseArray(JSON.toJSONString(list), OrganizeModel.class);
		for (OrganizeModel model : models) {
			model.setIcon("icon-ym icon-ym-tree-organization3");
		}
		List<SumTree<OrganizeModel>> trees = TreeDotUtils.convertListToTreeDot(models);
		List<OrganizeTreeVO> listVO = JSON.parseArray(JSON.toJSONString(trees), OrganizeTreeVO.class);
		// 将子节点全部删除
		Iterator<OrganizeTreeVO> iterator = listVO.iterator();
		while (iterator.hasNext()) {
			OrganizeTreeVO orananizeTreeVO = iterator.next();
			if (!"-1".equals(orananizeTreeVO.getParentId())) {
				iterator.remove();
			}
		}
		ListVO<OrganizeTreeVO> vo = new ListVO<>();
		vo.setList(listVO);
		return R.data(vo);
	}

	/**
	 * 获取组织信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取组织信息")
	@GetMapping("/{id}")
	public R<OrganizeInfoVO> info(@PathVariable("id") String id) throws DataException {
		OrganizeEntity entity = organizeService.getInfo(id);
		OrganizeInfoVO vo = JSON.parseObject(JSON.toJSONString(entity), OrganizeInfoVO.class);
		return R.data(vo);
	}

	/**
	 * 获取部门列表
	 *
	 * @param companyId
	 * @return
	 */
	@ApiOperation("获取部门列表")
	@GetMapping("/{companyId}/Department")
	public R<ListVO<OrganizeDepartListVO>> getListDepartment(@PathVariable("companyId") String companyId,
			Pagination pagination) {
		List<OrganizeEntity> dataAll = organizeService.getList();
		String parentId = dataAll.stream()
				.filter(t -> t.getId().equals(companyId))
				.findFirst()
				.orElse(new OrganizeEntity())
				.getId();
		List<OrganizeEntity> data = new ArrayList<>();
		// 将部门之下的子集搜索出来
		List<OrganizeEntity> dataCopy = dataAll.stream()
				.filter(t -> "department".equals(t.getCategory()))
				.collect(Collectors.toList());
		for (OrganizeEntity entity : dataCopy) {
			List<OrganizeEntity> data1 = new ArrayList<>();
			data1.add(entity);
			String id = JSON
					.parseArray(JSON.toJSONString(ListToTreeUtil.treeWhere(data1, dataAll)), OrganizeEntity.class)
					.stream()
					.filter(t -> companyId.equals(t.getId()))
					.findFirst()
					.orElse(new OrganizeEntity())
					.getId();
			if (parentId.equals(id)) {
				data.add(entity);
			}
		}
		if (!StrUtil.isEmpty(pagination.getKeyword())) {
			data = data.stream()
					.filter(t -> t.getFullName().contains(pagination.getKeyword())
							|| t.getEnCode().contains(pagination.getKeyword()))
					.collect(Collectors.toList());
		}
		List<OrganizeModel> models = JSON.parseArray(JSON.toJSONString(data), OrganizeModel.class);

		// 给部门经理赋值
		for (OrganizeModel model : models) {
			if (!StrUtil.isEmpty(model.getManager())) {
				UserEntity entity = userService.getById(model.getManager());
				if (entity != null) {
					model.setManager(entity.getRealName() + "/" + entity.getAccount());
				}
			}
		}
		List<SumTree<OrganizeModel>> trees = TreeDotUtils.convertListToTreeDot(models);
		// 去掉子公司的部门
		trees = trees.stream().filter(t -> parentId.equals(t.getParentId())).collect(Collectors.toList());
		List<OrganizeDepartListVO> listvo = JSON.parseArray(JSON.toJSONString(trees), OrganizeDepartListVO.class);
		ListVO<OrganizeDepartListVO> vo = new ListVO<>();
		vo.setList(listvo);
		return R.data(vo);
	}

	/**
	 * 获取部门下拉框列表
	 *
	 * @return
	 */
	@ApiOperation("获取部门下拉框列表")
	@GetMapping("/Department/Selector")
	public R<ListVO<OrganizeDepartSelectorListVO>> getListDepartment() {
		List<OrganizeEntity> allList = organizeService.getList();
		List<OrganizeEntity> data = allList.stream()
				.filter(t -> "0".equals(String.valueOf(t.getIsDeleted())))
				.collect(Collectors.toList());
		List<OrganizeModel> models = JSON.parseArray(JSON.toJSONString(data), OrganizeModel.class);
		for (OrganizeModel model : models) {
			if ("department".equals(model.getType())) {
				model.setIcon("icon-ym icon-ym-tree-department1");
			} else if ("company".equals(model.getType())) {
				model.setIcon("icon-ym icon-ym-tree-organization3");
			}
		}
		List<SumTree<OrganizeModel>> trees = TreeDotUtils.convertListToTreeDot(models);
		List<OrganizeDepartSelectorListVO> listVO = JSON.parseArray(JSON.toJSONString(trees),
				OrganizeDepartSelectorListVO.class);

		// 将子节点全部删除
		Iterator<OrganizeDepartSelectorListVO> iterator = listVO.iterator();
		while (iterator.hasNext()) {
			OrganizeDepartSelectorListVO organizeDepartSelectorListVO = iterator.next();
			if (!"-1".equals(organizeDepartSelectorListVO.getParentId())) {
				iterator.remove();
			}
		}

		ListVO<OrganizeDepartSelectorListVO> vo = new ListVO<>();
		vo.setList(listVO);
		return R.data(vo);
	}

	/**
	 * 获取部门信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取部门信息")
	@GetMapping("/Department/{id}")
	public R<OrganizeDepartInfoVO> infoDepartment(@PathVariable("id") String id) throws DataException {
		OrganizeEntity entity = organizeService.getInfo(id);
		OrganizeDepartInfoVO vo = JSON.parseObject(JSON.toJSONString(entity), OrganizeDepartInfoVO.class);
		return R.data(vo);
	}
}
