package org.springblade.controller;

import java.util.ArrayList;
import java.util.List;

import org.springblade.core.tool.api.R;
import org.springblade.model.dto.UserIdListVo;
import org.springblade.model.dto.UserIdModel;
import org.springblade.model.po.OrganizeEntity;
import org.springblade.model.po.UserDept;
import org.springblade.model.po.UserEntity;
import org.springblade.model.vo.ListVO;
import org.springblade.model.vo.UserSelectorVO;
import org.springblade.service.OrganizeService;
import org.springblade.service.UserService;
import org.springblade.service.UserDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.lang.Dict;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 数据处理
 *
 * @author william
 * @date 2022-07-14 10:57:53
 */
@Api(tags = "用户管理", value = "Users")
@RequestMapping("/paas/permission/Users")
@RestController
@SuppressWarnings("rawtypes")
public class UserController {
	@Autowired
	private UserService userService;

	@Autowired
	private OrganizeService organizeService;

	@Autowired
	private UserDeptService userDeptService;

	@ApiOperation("获取用户下拉框列表")
	@PostMapping("/ImUser/Selector/{organizeId}")
	public R selector(@PathVariable("organizeId") String organizeId) {
		List<UserSelectorVO> jsonToList = new ArrayList<>();
		if (StrUtil.isNotEmpty(organizeId)) {
			if (organizeId.equalsIgnoreCase("-1")) {
				// 返回公司基本信息
				UserSelectorVO userSelectorVO = new UserSelectorVO();
				userSelectorVO.setId("0");
				userSelectorVO.setParentId("-1");
				userSelectorVO.setType("company");
				userSelectorVO.setFullName("联通在线");
				userSelectorVO.setEnabledMark(1);
				userSelectorVO.setIcon("icon-ym icon-ym-tree-organization3");
				userSelectorVO.setHasChildren(true);
				userSelectorVO.setIsLeaf(false);
				jsonToList.add(userSelectorVO);
			} else {
				// 查询部门或者用户
				// 先查部门，没有部门则查用户
				LambdaQueryWrapper<OrganizeEntity> deptWrapper = Wrappers.<OrganizeEntity>lambdaQuery();
				deptWrapper.eq(OrganizeEntity::getParentId, Long.parseLong(organizeId));
				List<OrganizeEntity> deptList = organizeService.list(deptWrapper);
				if (deptList == null || deptList.isEmpty()) {
					// 没有部门，则查询用户
					LambdaQueryWrapper<UserDept> userDeptWrapper = Wrappers.<UserDept>lambdaQuery();
					userDeptWrapper.eq(UserDept::getDeptId, Long.parseLong(organizeId));
					List<UserDept> userDeptList = userDeptService.list(userDeptWrapper);
					if (userDeptList != null && !userDeptList.isEmpty()) {
						for (UserDept userDept : userDeptList) {
							Long userId = userDept.getUserId();
							UserEntity user = userService.getById(userId);
							if (user != null) {
								UserSelectorVO userSelectorVO = new UserSelectorVO();
								userSelectorVO.setId(String.valueOf(user.getId()));
								userSelectorVO.setParentId(organizeId);
								userSelectorVO.setType("user");
								userSelectorVO.setFullName(user.getRealName() + "/" + user.getAccount());
								userSelectorVO.setIcon("icon-ym icon-ym-tree-user2");
								userSelectorVO.setIsLeaf(true);
								jsonToList.add(userSelectorVO);
							}
						}
					}
				} else {
					deptList.forEach(dept -> {
						UserSelectorVO userSelectorVO = new UserSelectorVO();
						userSelectorVO.setId(String.valueOf(dept.getId()));
						userSelectorVO.setParentId(String.valueOf(dept.getParentId()));
						userSelectorVO.setType("department");
						userSelectorVO.setFullName(dept.getFullName());
						userSelectorVO.setEnabledMark(1);
						userSelectorVO.setIcon("icon-ym icon-ym-tree-department1");
						if (organizeService.hasChild(organizeId) || organizeService.hasUser(organizeId)) {
							userSelectorVO.setIsLeaf(false);
							userSelectorVO.setHasChildren(true);
							jsonToList.add(userSelectorVO);
						} else {
							userSelectorVO.setIsLeaf(true);
							userSelectorVO.setHasChildren(false);
						}
					});
				}
			}
		}

		return R.data(Dict.create().set("list", jsonToList));
	}

	/**
	 * 获取用户基本信息
	 *
	 * @param userIdModel
	 * @return
	 */
	@ApiOperation("获取用户基本信息")
	@PostMapping("/getUserList")
	public R<Object> getUserList(@RequestBody UserIdModel userIdModel) {
		List<UserIdListVo> list = new ArrayList<>();
		for (String userId : userIdModel.getUserId()) {
			UserEntity entity = userService.getInfo(userId);
			if (entity == null) {
				break;
			}
			UserIdListVo vo = JSON.parseObject(JSON.toJSONString(entity), UserIdListVo.class);
			vo.setFullName(entity.getRealName() + "/" + entity.getAccount());
			list.add(vo);
		}
		ListVO<UserIdListVo> listVO = new ListVO<>();
		listVO.setList(list);
		return R.data(listVO);
	}
}
