package org.springblade.controller;

import java.util.Date;
import java.util.List;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.model.po.DictionaryDataEntity;
import org.springblade.model.po.DictionaryTypeEntity;
import org.springblade.service.DictionaryDataService;
import org.springblade.service.DictionaryTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.extra.pinyin.PinyinUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @author William
 * @date 2022年12月20日 下午1:44:59
 */
@Api(tags = "表单分类", value = "cate")
@RestController
@RequestMapping("/paas/visualdev/cate")
public class VisualdevCateController {
	@Autowired
	private DictionaryDataService dictDataService;

	@Autowired
	private DictionaryTypeService dictTypeService;

	@ApiOperation("列表")
	@GetMapping("/list")
	public R<List<DictionaryDataEntity>> list() {
		List<DictionaryDataEntity> cateList = dictDataService.getListForVisualdev();
		return R.data(cateList);
	}

	@ApiOperation("增加")
	@PostMapping("/add")
	public R<String> add(String name) {
		DictionaryDataEntity data = new DictionaryDataEntity();
		DictionaryTypeEntity type = dictTypeService.getInfoByEnCode("webDesign");
		data.setDictionaryTypeId(type.getId());
		data.setParentId(type.getId());
		data.setFullName(name);
		data.setEnCode(PinyinUtil.getPinyin(name).toUpperCase());
		data.setSimpleSpelling(PinyinUtil.getFirstLetter(name, "").toUpperCase());
		data.setCreatorTime(new Date());
		data.setCreatorUserId(String.valueOf(SecureUtil.getUserId()));
		return R.status(dictDataService.save(data));
	}

	@ApiOperation("更新")
	@PostMapping("/update")
	public R<String> update(String id, String name) {
		DictionaryDataEntity data = dictDataService.getById(id);
		if (data == null) {
			return R.fail("数据不存在");
		}
		DictionaryTypeEntity type = dictTypeService.getInfoByEnCode("webDesign");
		data.setDictionaryTypeId(type.getId());
		data.setParentId(type.getId());
		data.setFullName(name);
		data.setEnCode(PinyinUtil.getPinyin(name).toUpperCase());
		data.setSimpleSpelling(PinyinUtil.getFirstLetter(name, "").toUpperCase());
		data.setLastModifyTime(new Date());
		data.setLastModifyUserId(String.valueOf(SecureUtil.getUserId()));
		return R.status(dictDataService.updateById(data));
	}

	@ApiOperation("详情")
	@GetMapping("/info")
	public R<DictionaryDataEntity> info(String id) {
		return R.data(dictDataService.getById(id));
	}

	@ApiOperation("删除")
	@GetMapping("/delete")
	public R<String> delete(String id) {
		return R.status(dictDataService.removeById(id));
	}
}
