package org.springblade.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.bi.client.CubeDataClient;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.model.po.ResetSerialnumber;
import org.springblade.model.po.VisualdevDataPermission;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.ResetSerialnumberService;
import org.springblade.service.VisualdevDataPermissionService;
import org.springblade.service.VisualdevService;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.UserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 表单设计
 *
 * @author William
 * @date 2022年12月13日 下午6:00:05
 */
@Api(tags = "表单设计", value = "Base")
@RestController
@RequestMapping("/paas/visualdev/Base")
public class VisualdevController {

    @Resource
    ResetSerialnumberService resetSerialnumberService;
    @Autowired
    private VisualdevService visualService;
    @Autowired
    private VisualdevDataPermissionService dataPermissionService;
    @Autowired
    private UserClient userClient;
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private CubeDataClient cubeDataClient;

    @ApiOperation("分页查询")
    @GetMapping("/page")
    public R<IPage<VisualdevEntity>> page(Integer current, Integer size, String cateId, String name) {
        LambdaQueryWrapper<VisualdevEntity> wrapper = Wrappers.<VisualdevEntity>lambdaQuery();
        if (StrUtil.isNotBlank(name)) {
            wrapper.like(VisualdevEntity::getFullName, name);
        }
        if (StrUtil.isNotBlank(cateId)) {
            wrapper.eq(VisualdevEntity::getCategory, cateId);
        }
        wrapper.eq(VisualdevEntity::getType, 1);
        wrapper.orderByDesc(VisualdevEntity::getCreatorTime);
        wrapper.select(VisualdevEntity::getId,
                VisualdevEntity::getFullName,
                VisualdevEntity::getCreatorTime,
                VisualdevEntity::getCreatorUser,
                VisualdevEntity::getLastModifyTime,
                VisualdevEntity::getLastModifyUser,
                VisualdevEntity::getState,
                VisualdevEntity::getEnCode);
        Query query = new Query();
        query.setCurrent(current);
        query.setSize(size);
        IPage<VisualdevEntity> pageData = visualService.page(Condition.getPage(query), wrapper);
        if (pageData != null && CollUtil.isNotEmpty(pageData.getRecords())) {
            pageData.getRecords().forEach(form -> {
                UserModel creator = userClient.getById(Long.parseLong(form.getCreatorUser()));
                form.setCreatorUser(creator != null ? creator.getRealName() : "");

                if (form.getLastModifyUser() != null) {
                    UserModel modifer = userClient.getById(Long.parseLong(form.getLastModifyUser()));
                    form.setLastModifyUser(modifer != null ? modifer.getRealName() : "");
                }
            });
        }
        return R.data(pageData);
    }

    @ApiOperation("详情")
    @GetMapping("/{id}")
    public R<VisualdevEntity> info(@PathVariable("id") String id) {
        VisualdevEntity entity = visualService.getById(id);
        return R.data(entity);
    }

    @ApiOperation("复制")
    @PostMapping("/{id}/Actions/Copy")
    public R<String> copyInfo(@PathVariable("id") String id) {
        VisualdevEntity entity = visualService.getById(id);
        entity.setState(0);
        entity.setFullName(entity.getFullName() + "_副本");
        entity.setId(IdUtil.simpleUUID());
        visualService.save(entity);
        return R.success("复制成功");
    }

    @ApiOperation("更新状态")
    @PutMapping("/{id}/Actions/State")
    public R<String> update(@PathVariable("id") String id) {
        VisualdevEntity entity = visualService.getById(id);
        if (entity != null) {
            if (entity.getState() == 1) {
                entity.setState(0);
            } else {
                entity.setState(1);
            }
            boolean flag = visualService.updateById(entity);
            if (flag == false) {
                return R.fail("更新失败，任务不存在");
            }
        }
        return R.success("更新成功");
    }

    @ApiOperation("新建")
    @PostMapping()
    public R<String> createV2(@RequestBody VisualdevEntity entity) {
        if (StrUtil.isEmpty(entity.getId())) {
            entity.setId(IdUtil.fastSimpleUUID());
        }
        entity.setSortCode(0L);
        entity.setCreatorTime(new Date());
        entity.setCreatorUser(String.valueOf(SecureUtil.getUserId()));
        boolean flag = visualService.save(entity);
        VisualdevDataPermission defPermission = null;
        if (flag) {
            LambdaQueryWrapper<VisualdevDataPermission> wrapper = Wrappers.lambdaQuery(VisualdevDataPermission.class)
                    .eq(VisualdevDataPermission::getIsDefault, 1)
                    .eq(VisualdevDataPermission::getVisualdevId, entity.getId());
            defPermission = dataPermissionService.getOne(wrapper);
            if (defPermission == null) {
                // 该表单不存在默认权限，则添加一条默认权限组
                String permJson = "[{\"permKey\":\"add\",\"permName\":\"新增\",\"permValue\":1},{\"permKey\":\"download\",\"permName\":\"导出\",\"permValue\":1},{\"permKey\":\"batchUpload\",\"permName\":\"批量导入\",\"permValue\":0},{\"permKey\":\"batchRemove\",\"permName\":\"批量删除\",\"permValue\":0},{\"permKey\":\"edit\",\"permName\":\"编辑\",\"permValue\":0},{\"permKey\":\"remove\",\"permName\":\"删除\",\"permValue\":1},{\"permKey\":\"detail\",\"permName\":\"详情\",\"permValue\":0}]";
                defPermission = new VisualdevDataPermission();
                defPermission.setCreateTime(new Date());
                defPermission.setCreateUserId(SecureUtil.getUserId());
                defPermission.setVisualdevId(entity.getId());
                defPermission.setIsDefault(1);
                defPermission.setPermJson(permJson);
                defPermission.setDataPermId(0);
                defPermission.setVisualdevPermName("默认权限组");
                dataPermissionService.save(defPermission);
            }
        }
        R<Object> r = R.success("新建成功");
        r.setData(defPermission);

        cubeDataClient.changeCubeDataStatus(Integer.valueOf(entity.getDbLinkId()), true);
        if (StrUtil.isNotBlank(entity.getSecondaryDbLinkId())){
            JSONArray objects = JSONUtil.parseArray(entity.getSecondaryDbLinkId());
            for (Object object : objects) {
                cubeDataClient.changeCubeDataStatus(Integer.valueOf(object+""), true);
            }
        }

        return R.success("新建成功");
    }


    @ApiOperation("修改")
    @PostMapping("/update")
    public R<String> update(@RequestBody VisualdevEntity entity) {
        entity.setLastModifyTime(new Date());
        entity.setLastModifyUser(String.valueOf(SecureUtil.getUserId()));
        boolean flag = visualService.updateById(entity);
        return R.status(flag);
    }

    @ApiOperation("修改")
    @PostMapping("/serialnumber/save")
    public R<String> update(@RequestBody ResetSerialnumber entity) {
        int st = entity.getStartValue();
        entity.setStartValue(st);
        entity.setCount(Long.valueOf(st + ""));

        entity.setCreateTime(new Date());
        entity.setCreateUserId(SecureUtil.getUserId());
        boolean flag = false;
        QueryWrapper<ResetSerialnumber> wrapper = new QueryWrapper<ResetSerialnumber>().eq("model_id", entity.getModelId());
        ResetSerialnumber one = resetSerialnumberService.getOne(wrapper);
        if (one == null) {
            flag = resetSerialnumberService.save(entity);
        }
        if (one != null) {
            one.setResetCycle(entity.getResetCycle());
            one.setStartValue(entity.getStartValue());
            flag = resetSerialnumberService.updateById(one);
        }
        return R.status(flag);
    }

    @ApiOperation("删除")
    @PostMapping("/delete")
    public R<String> delete(@RequestParam String id) {
        VisualdevEntity entity = visualService.getById(id);
        if (entity != null) {
            visualService.removeById(id);
            return R.success("删除成功");
        }
        return R.fail("删除失败，数据不存在");
    }

    @ApiOperation("数据集的字段可选性")
    @GetMapping("/fieldOption")
    public R<Kv> fieldOption(String cubeId) {
        return R.data(dataSourceService.fieldOption(cubeId));
    }

}
