package org.springblade.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.Func;
import org.springblade.enums.FunctionAttrEnum;
import org.springblade.enums.FunctionTypeEnum;
import org.springblade.model.po.VisualdevFunction;
import org.springblade.service.VisualdevFunctionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/paas/visualdev/function")
@Api(value = "表单函数接口", tags = "表单函数接口")
public class VisualdevFunctionController extends BladeController {

    @Autowired
    private VisualdevFunctionService functionService;

    @GetMapping("/list")
    @ApiOperation(value = "函数列表")
    public R<List<VisualdevFunction>> list( FunctionAttrEnum funcAttr, String name){
        QueryWrapper<VisualdevFunction> queryWrapper = Wrappers.<VisualdevFunction>query();
//		queryWrapper.in("ds_type", dsType.getValue(), -1); //数据源类型【-1 代表通用】
        queryWrapper.lambda()
                .like(StrUtil.isNotBlank(name), VisualdevFunction::getName, name)
                .eq(funcAttr != null, VisualdevFunction::getFuncAttr, funcAttr)
                .and(t -> t.eq(VisualdevFunction::getFuncType, FunctionTypeEnum.system).or().eq(VisualdevFunction::getTenantId,getUser().getTenantId()))
                .orderByDesc(VisualdevFunction::getSortNum);
        return R.data(functionService.list(queryWrapper));
    }

    /**
     *
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "新增修改函数")
    public R<String> doSubmit(@RequestBody VisualdevFunction visualdevFunction){
        visualdevFunction.setCreateTime(new Date());
        return R.status(functionService.saveOrUpdate(visualdevFunction));
    }

    /**
     *
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除函数")
    public R<String> doDelete(@ApiParam(value = "主键", required = true) @RequestParam String ids){
        return R.status(functionService.removeByIds(Func.toIntList(ids)));
    }

    /**
     *
     * @return
     */
    @GetMapping("/funcAttrList")
    @ApiOperation(value = "函数属性列表")
    public R<List<Kv>> funcAttrList() {
        List<Kv> dataList = new ArrayList<>();
        for(FunctionAttrEnum funcAttr : FunctionAttrEnum.values()) {
            dataList.add(Kv.create().set("value", funcAttr.getValue()).set("name", funcAttr.name()).set("message", funcAttr.getMessage()));
        }
        return R.data(dataList);
    }


}
