package org.springblade.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.extra.servlet.ServletUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.database.FormDataService;
import org.springblade.db.excel.export.ExportService;
import org.springblade.db.excel.import_.ExcelFormParam;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.enums.OperateType;
import org.springblade.model.dto.BatchId;
import org.springblade.model.dto.DataExportParam;
import org.springblade.model.dto.PermJsonDto;
import org.springblade.model.dto.VisualdevDataDto;
import org.springblade.model.po.*;
import org.springblade.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 表单数据
 *
 * @author William
 * @date 2022年12月13日 下午5:59:55
 */
@Api(tags = "表单数据", value = "OnlineDev")
@RestController
@RequestMapping("/paas/visualdev/OnlineDev")
@Slf4j
public class VisualdevModelDataController extends BladeController {

    @Resource
    ResetSerialnumberService resetSerialnumberService;
    @Autowired
    private VisualdevService visualdevService;
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private FormDataService formDataService;
    @Autowired
    private VisualdevDataPermissionService dataPermissionService;
    @Autowired
    private DataPermissionUserService dataPermissionUserService;
    @Autowired
    private FormDataLogService logService;
    @Autowired
    private DataExportTaskService dataExportTaskService;
    @Value("${blade.file.uploadPath}")
    private String uploadDir;
    @Autowired
    private ExportService exportService;
    @Autowired
    private DataImportLogService importLogService;
    @Autowired
    private DataRecycleService recycleService;
    @Autowired
    private BladeRedis redisService;

    public static void main(String[] args) {
        File file = new File("D:\\www\\upload\\1.txt");
        if (file == null || !file.exists()) {
            System.out.println(11111);
        }
    }

    @ApiOperation("表单 - 列表表单配置JSON(详情)")
    @GetMapping("/{modelId}/Config")
    public R<VisualdevEntity> getData(@PathVariable("modelId") String modelId) {
        VisualdevEntity entity = visualdevService.getById(modelId);
        if (entity == null) {
            return R.fail("功能不存在");
        }
        entity.setIdKey(dataSourceService.getPrimaryKeyCubeIdOfMainTable(entity));
        return R.data(entity);
    }

    @ApiOperation("表单 - 列表配置JSON")
    @GetMapping("/{modelId}/ColumnData")
    public R<Kv> getColumnData(@PathVariable("modelId") String modelId) {
        VisualdevEntity entity = visualdevService.getById(modelId);
        return R.data(Kv.create().set("columnData", entity.getColumnData()));
    }

    // @ApiOperation("数据操作 - 分页查询")
    // @PostMapping("/{modelId}/List")
    // public ActionResult<?> list(@PathVariable("modelId") String modelId,
    // @RequestBody PaginationModel paginationModel)
    // throws ParseException, IOException, SQLException, DataException {
    // VisualdevEntity visualdevEntity = visualdevService.getInfo(modelId);
    // return formDataWithTableService.list(visualdevEntity, paginationModel);
    // }

    @ApiOperation("表单 - 表单配置JSON")
    @GetMapping("/{modelId}/FormData")
    public R<Kv> getFormData(@PathVariable("modelId") String modelId) {
        VisualdevEntity entity = visualdevService.getById(modelId);
        return R.data(Kv.create().set("formData", entity.getFormData()));
    }

    @ApiOperation("数据 - 添加")
    @PostMapping("/{modelId}")
    @Transactional
    public R<Object> create(@PathVariable("modelId") String modelId, @RequestBody VisualdevDataDto formData, HttpServletRequest request) throws SQLException{
        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        //查询新增操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("add".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", modelId);
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getFormData().getBytes())));
            //数据 春说暂时先注释用不上
//			paramMap.put("dataJson",new String(Base64.getEncoder().encode(formData.getData().getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.add);


            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {
            return formDataService.add(modelId, formData.getData());
        }

    }

    @ApiOperation("数据 - 添加")
    @PostMapping("/v2/{modelId}")
    @Transactional(rollbackFor = Exception.class)
    public R<Object> createV2(@PathVariable("modelId") String modelId, @RequestBody List<VisualdevDataDto> formData, HttpServletRequest request) throws SQLException{
        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        //查询新增操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("add".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", modelId);
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getFormData().getBytes())));
            //数据 春说暂时先注释用不上
//			paramMap.put("dataJson",new String(Base64.getEncoder().encode(formData.getData().getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.add);


            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {
            List<Object> arrayList = Lists.newArrayList();
            for (VisualdevDataDto formDatum : formData) {
                String formDatumMap = resetSerialnumberService.generateSerialNumber(modelId, formDatum);
                R<Object> objectR = formDataService.add(modelId, formDatumMap);
                arrayList.add(objectR.getData());
            }
            return R.data(arrayList);
        }

    }


    /**
     * @return
     */
    @PostMapping("/doAdd")
    @ApiOperation(value = "执行添加操作")
    @Transactional
    public R<Object> doAdd(String busId, String handleStatus, String data) throws UnsupportedEncodingException ,SQLException{
        if (StringUtil.isNotBlank(data)) {
            data = new String(Base64.getDecoder().decode(data.replaceAll(" +", "+")), "utf-8");
        }
        VisualdevEntity formInfo = visualdevService.getById(busId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        return formDataService.add(busId, data);
    }

    @ApiOperation("数据 - Excel导入")
    @PostMapping("/createBatch")
    @Transactional(rollbackFor = Exception.class)
    public R<Kv> createBatch(@RequestBody ExcelFormParam params, HttpServletRequest request) throws SQLException {
        if (params == null) {
            return R.fail("缺少参数");
        }

        if (CollUtil.isEmpty(params.getList())) {
            return R.fail("缺少字段和索引映射");
        }

        String filePath = uploadDir + params.getFile().replace("uploads/", "");
        File excelFile = new File(filePath);
        if (excelFile == null || !excelFile.exists()) {
            return R.fail("Excel文件不存在");
        }

        String modelId = params.getModelId();
        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }

        //查询导出操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, params.getModelId()));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("batchUpload".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(JSON.toJSONString(params).getBytes())));
//			params.setFile(filePath);
            paramMap.put("busId", JSON.toJSONString(params));
            //表单结构
            paramMap.put("busJson", "");

            paramMap.put("flowId", flowId);
            paramMap.put("isTable", 2);//excel
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.batchUpload);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {

                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {//不用走流程，直接导出
            CubeDataModel cubeData = dataSourceService.getCubeData(formInfo.getDbLinkId());
            if (params.isTruncate()) {
                // 先清空数据
                formDataService.truncate(cubeData, formInfo);
            }
            List<ExcelFieldIndex> indexList = params.getList();
            boolean withHeader = params.getWithHeader();
            R<Kv> ret = formDataService.batchAdd(formInfo, excelFile, indexList, withHeader);
            long total = ret.getData().getLong("count");
            float takeUpTime = ret.getData().getFloat("takeUpTime");
            String keyStart = ret.getData().getStr("keyStart");
            String keyEnd = ret.getData().getStr("keyEnd");
            importLogService.saveLog(params, total, cubeData.getId(), takeUpTime, keyStart, keyEnd);
            return ret;
        }

    }

    /**
     * @return
     */
    @PostMapping("/doBatchImport")
    @ApiOperation(value = "执行批量导入操作")
    public R<Object> doBatchImport(String busId) throws SQLException {
        ExcelFormParam param = JSON.parseObject(busId, ExcelFormParam.class);
        VisualdevEntity formInfo = visualdevService.getById(param.getModelId());
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        CubeDataModel cubeData = dataSourceService.getCubeData(formInfo.getDbLinkId());
        if (param.isTruncate()) {
            // 先清空数据
            formDataService.truncate(cubeData, formInfo);
        }
        String filePath = uploadDir + param.getFile().replace("uploads/", "");
        File excelFile = new File(filePath);
        if (excelFile == null || !excelFile.exists()) {
            return R.fail("Excel文件不存在");
        }
        List<ExcelFieldIndex> indexList = param.getList();
        boolean withHeader = param.getWithHeader();
        R<Kv> ret = formDataService.batchAdd(formInfo, excelFile, indexList, withHeader);
        long total = ret.getData().getLong("count");
        float takeUpTime = ret.getData().getFloat("takeUpTime");
        String keyStart = ret.getData().getStr("keyStart");
        String keyEnd = ret.getData().getStr("keyEnd");
        importLogService.saveLog(param, total, cubeData.getId(), takeUpTime, keyStart, keyEnd);
        return R.success("执行批量导入操作成功");
    }

    @ApiOperation("数据 - 更新")
    @PutMapping("/{modelId}/{id}")
    public R<?> update(@PathVariable("modelId") String modelId, @PathVariable("id") String id,
                       @RequestBody VisualdevDataDto formData, HttpServletRequest request)
            throws SQLException {
        if (StrUtil.isBlank(id)) {
            return R.fail("缺少ID，无法修改");
        }
        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }

        if (redisService.exists(modelId + "_" + id)) {
            return R.fail("正在审核中,无法编辑");
        }
        //查询新增操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("edit".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", modelId + "_" + id);
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getFormData().getBytes())));
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(JSONObject.toJSONString(formData).getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.edit);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                redisService.set(modelId + "_" + id, "1");
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {
            R<?> ret = formDataService.update(modelId, id, formData.getData() + "");

            if (ret.getCode() == 200) {
                Map<String, Object> oldData = null;
                try {
                    oldData = JSON.parseObject(formData.getOldData());
                } catch (Exception e) {
                }
                Map<String, Object> newData = null;
                try {
                    newData = JSON.parseObject(formData.getData() + "");
                } catch (Exception e) {
                }
                logService.saveLog(modelId, id, oldData, newData);
            }
            return ret;

        }
    }

    @ApiOperation("数据 - 更新")
    @PutMapping("/v2/{modelId}/{id}")
    @Transactional(rollbackFor = Exception.class)
    public R<?> update22(@PathVariable("modelId") String modelId, @PathVariable("id") String id,
                         @RequestBody List<VisualdevDataDto> formDatas, HttpServletRequest request)
            throws SQLException {
        if (StrUtil.isBlank(id)) {
            return R.fail("缺少ID，无法修改");
        }
        if (CollUtil.isEmpty(formDatas)) {
            return R.fail("formDatas不能为空，无法修改");
        }
        VisualdevDataDto formData = formDatas.get(0);
        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }

        if (redisService.exists(modelId + "_" + id)) {
            return R.fail("正在审核中,无法编辑");
        }
        //查询新增操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("edit".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", modelId + "_" + id);
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getFormData().getBytes())));
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(JSONObject.toJSONString(formData).getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.edit);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                redisService.set(modelId + "_" + id, "1");
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {
            R<?> ret = formDataService.update(modelId, id, formData.getData() + "");

            if (ret.getCode() == 200) {
                Map<String, Object> oldData = null;
                try {
                    oldData = JSON.parseObject(formData.getOldData());
                } catch (Exception e) {
                }
                Map<String, Object> newData = null;
                try {
                    newData = JSON.parseObject(formData.getData() + "");
                } catch (Exception e) {
                }
                logService.saveLog(modelId, id, oldData, newData);
            }
            return ret;

        }
    }

    @ApiOperation("数据 - 批量更新")
    @PostMapping("/batchUpdate/{modelId}")
    public R<?> batchUpdate(@PathVariable("modelId") String modelId,
                            @RequestBody List<VisualdevDataDto> formDataList, HttpServletRequest request)
            throws SQLException {

        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        if (formDataList != null && formDataList.size() > 0) {
            R<?> ret = formDataService.batchUpdate(modelId, formDataList);
            if (ret.getCode() == 200) {
                logService.batchSaveLog(modelId, formDataList);
            } else {
                return R.fail("批量编辑失败!");
            }

        }
        return R.success("批量编辑成功!");

    }

    /**
     * @return
     */
    @PostMapping("/doUpdate")
    @ApiOperation(value = "执行修改操作")
    public void doUpdate(String busId, String handleStatus, String data) {
        try {
            String[] array = busId.split("_");
            String modelId = array[0];
            String id = array[1];
            if (StringUtil.isNotBlank(data)) {
                data = new String(Base64.getDecoder().decode(data.replaceAll(" +", "+")), "utf-8");
            }
            VisualdevDataDto formData = JSON.parseObject(data, VisualdevDataDto.class);
            R<?> ret = formDataService.update(modelId, id, formData.getData() + "");
            if (ret.getCode() == 200) {
                Map<String, Object> oldData = null;
                try {
                    oldData = JSON.parseObject(formData.getOldData());
                } catch (Exception e) {
                }
                Map<String, Object> newData = null;
                try {
                    newData = JSON.parseObject(formData.getData() + "");
                } catch (Exception e) {
                }
                logService.saveLog(modelId, id, oldData, newData);
            }
        } catch (Exception e) {
            log.error("执行表单修改操作报错：" + e.getMessage());
        } finally {
            redisService.del(busId);
        }
    }

    @ApiOperation("数据 - 删除")
    @DeleteMapping("/{modelId}/{id}")
    public R<String> delete(@PathVariable("modelId") String modelId, @PathVariable("id") String id, @RequestParam String dataJson, HttpServletRequest request)
            throws SQLException, UnsupportedEncodingException {
        if (StrUtil.isBlank(id)) {
            return R.fail("缺少ID，无法删除");
        }

        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }
        if (redisService.exists(modelId + "_" + id)) {
            return R.fail("正在审核中,无法删除");
        }

        //查询删除操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("remove".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", modelId + "_" + id);
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getFormData().getBytes())));
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(dataJson.getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.remove);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
//			String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            String resBody = HttpUtil.createPost("http://127.0.0.1:6464/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                redisService.set(modelId + "_" + id, "1");
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {//不用走流程，直接删除
            LinkedHashMap<String, List<JSONObject>> data = formDataService.selectForRecycle(modelId, id);
            if (data != null && !data.isEmpty()) {
                recycleService.save(modelId, data);
            }
            List<String> idList = new ArrayList<>();
            idList.add(id);
            return formDataService.delete(modelId, idList);
        }
    }

    /**
     * @return
     */
    @PostMapping("/doDelete")
    @ApiOperation(value = "执行删除操作")
    public R<Object> doDelete(String busId, String handleStatus, String data) {
        try {
            if (StrUtil.isNotBlank(busId)) {
                String[] array = busId.split("_");
                String modelId = array[0];
                String id = array[1];
                if (StrUtil.isBlank(id)) {
                    return R.fail("缺少ID，无法删除");
                }
                VisualdevEntity formInfo = visualdevService.getById(modelId);
                if (formInfo == null) {
                    return R.fail("表单不存在");
                }
                LinkedHashMap<String, List<JSONObject>> map = formDataService.selectForRecycle(modelId, id);
                if (map != null && !map.isEmpty()) {
                    recycleService.save(modelId, map);
                }
                List<String> idList = new ArrayList<>();
                idList.add(id);
                formDataService.delete(modelId, idList);
                redisService.del(busId);
            } else {
                return R.fail("业务主键id不存在!");
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            redisService.del(busId);
        }
        return R.success("执行删除操作成功");
    }

    @ApiOperation("数据 - 批量删除")
    @PostMapping("/batchDelete/{modelId}")
    public R<String> batchDelete(@PathVariable("modelId") String modelId, @RequestBody BatchId idVo, HttpServletRequest request)
            throws SQLException {
        List<String> idList = idVo.getIds();
        if (CollUtil.isEmpty(idList)) {
            return R.fail("缺少ID，无法删除");
        }

        VisualdevEntity formInfo = visualdevService.getById(modelId);
        if (formInfo == null) {
            return R.fail("表单不存在");
        }

        if (idList.size() > 50) {
            return R.fail("批量删除最多支持50条");
        }
        for (String id : idList) {
            if (redisService.exists(modelId + "_" + id)) {
                return R.fail("正在审核中,无法删除");
            }
        }

//		boolean flag=false;
//		for (String id : idList){
//			if (redisService.exists(modelId+"_"+id)){
//				flag=true;
//				break;
//			}
//		}
//		if (flag){
//			return R.fail("正在审核中,无法删除");
//		}

        //查询删除操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("batchRemove".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            List<String> ids = idList.stream().map(x -> modelId + "_" + x).collect(Collectors.toList());
            paramMap.put("busId", StringUtil.join(ids, ","));
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getColumnData().getBytes())));
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(idVo.getDataJson().getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("isTable", 1);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.batchRemove);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                for (String id : ids) {
                    redisService.set(id, "1");
                }
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {//不用走流程，直接删除
            for (String id : idList) {
                LinkedHashMap<String, List<JSONObject>> data = formDataService.selectForRecycle(modelId, id);
                if (data != null && !data.isEmpty()) {
                    recycleService.save(modelId, data);
                }
            }
            return formDataService.delete(modelId, idList);
        }

    }

    /**
     * @return
     */
    @PostMapping("/doBatchDelete")
    @ApiOperation(value = "执行批量删除操作")
    public R<Object> doBatchDelete(String busId, String handleStatus) {
        try {
            List<String> idList = new ArrayList<>();
            List<String> list = Arrays.asList(busId.split(","));
            if (CollUtil.isEmpty(list)) {
                return R.fail("缺少ID，无法删除");
            }
            String[] array = list.get(0).split("_");
            String modelId = array[0];
            VisualdevEntity formInfo = visualdevService.getById(modelId);
            if (formInfo == null) {
                return R.fail("表单不存在");
            }
            for (String id : list) {
                redisService.del(id);
                idList.add(id.split("_")[1]);
                LinkedHashMap<String, List<JSONObject>> data = formDataService.selectForRecycle(modelId, id.split("_")[1]);
                if (data != null && !data.isEmpty()) {
                    recycleService.save(modelId, data);
                }
            }
            formDataService.delete(modelId, idList);

        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return R.success("执行删除操作成功");
    }

    @ApiOperation("数据 - 查询操作权限")
    @PostMapping("/getPerm/{modelId}")
    public R<Object> getPerm(@PathVariable("modelId") String modelId) {
        Long userId = getUser().getUserId();
        List<String> result = new ArrayList<>();
        // 查询用户所属权限
        List<DataPermissionUser> dataPermissionUsers = dataPermissionUserService
                .list(Wrappers.<DataPermissionUser>lambdaQuery().eq(DataPermissionUser::getUserId, userId));
        if (dataPermissionUsers != null && dataPermissionUsers.size() > 0) {
            for (DataPermissionUser dataPermissionUser : dataPermissionUsers) {
                VisualdevDataPermission visualdevDataPermission = dataPermissionService.getOne(Wrappers
                        .<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, modelId)
                        .eq(VisualdevDataPermission::getDataPermId, dataPermissionUser.getDataPermId()));
                if (visualdevDataPermission != null) {
                    String permJson = visualdevDataPermission.getPermJson();
                    List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                    for (PermJsonDto permJsonDto : permJsonDtos) {
                        Integer value = permJsonDto.getPermValue();
                        if (!result.contains(permJsonDto.getPermKey()) && value == 1) {
                            result.add(permJsonDto.getPermKey());
                        }
                    }
                }
            }
        }
        if (result.size() == 0) {
            // 取默认权限
            VisualdevDataPermission visualdevDataPermission = dataPermissionService.getOne(
                    Wrappers.<VisualdevDataPermission>lambdaQuery()
                            .eq(VisualdevDataPermission::getVisualdevId, modelId)
                            .eq(VisualdevDataPermission::getIsDefault, 1));
            if (visualdevDataPermission != null) {
                String permJson = visualdevDataPermission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    Integer value = permJsonDto.getPermValue();
                    if (!result.contains(permJsonDto.getPermKey()) && value == 1) {
                        result.add(permJsonDto.getPermKey());
                    }
                }
            }

        }
        return R.data(result);
    }

    @ApiOperation("数据 - 导出")
    @PostMapping("/export")
    public R<DataExportTask> export(@RequestBody DataExportParam param, HttpServletRequest request) {
        if (StrUtil.isBlank(param.getModelId())) {
            return R.fail("表单ID不能为空");
        }
        if (StrUtil.isBlank(param.getTaskName())) {
            return R.fail("任务名称不能为空");
        }
        if (CollUtil.isEmpty(param.getColumnIdList())) {
            return R.fail("要导出的字段ID不能为空");
        }
        if (!param.isAll() && CollUtil.isEmpty(param.getDataIdList())) {
            return R.fail("数据ID不能为空");
        }
        BladeUser user = getUser();
        VisualdevEntity formInfo = visualdevService.getById(param.getModelId());
        long taskId = IdWorker.getId();
        DataExportTask task = DataExportTask.builder()
                .id(taskId)
                .tenantId(user.getTenantId())
                .modelId(param.getModelId())
                .cubeId(Integer.parseInt(formInfo.getDbLinkId()))
                .name(param.getTaskName())
                .isAll(param.isAll() ? 1 : 0)
                .name(param.getTaskName())
                .createTime(new Date())
                .operator(SecureUtil.getNickName())
                .operatorId(SecureUtil.getUserId())
                .fileName(taskId + ".xlsx")
                .currentPage(param.getCurrentPage())
                .dataIdList(param.getDataIdList())
                .size(param.getSize())
                .paramList(JSON.toJSONString(param.getParamList()))
                .build();
        if (!param.isAll()) {
            task.setCount(param.getDataIdList().size());
        }
        // 组装导出规则json
        List<RelationField> fields = JSON.parseArray(formInfo.getTables(), RelationField.class);
        List<RelationField> rule = new ArrayList<>();
        List<Integer> columnIdList = param.getColumnIdList();
        for (int i = 0; i < columnIdList.size(); i++) {
            Integer columnId = columnIdList.get(i);
            fields.forEach(field -> {
                if (field.getFieldId() == columnId) {
                    rule.add(field);
                }
            });
        }
        task.setRule(JSON.toJSONString(rule));


        //查询导出操作是否绑定流程
        String flowId = "";
        List<VisualdevDataPermission> list = dataPermissionService.list(
                Wrappers.<VisualdevDataPermission>lambdaQuery()
                        .eq(VisualdevDataPermission::getVisualdevId, param.getModelId()));
        if (list != null && list.size() > 0) {
            for (VisualdevDataPermission permission : list) {
                String permJson = permission.getPermJson();
                List<PermJsonDto> permJsonDtos = JSON.parseArray(permJson, PermJsonDto.class);
                for (PermJsonDto permJsonDto : permJsonDtos) {
                    String permKey = permJsonDto.getPermKey();
                    if ("download".equals(permKey)) {
                        if (permJsonDto.getPermValue() == 1 && StringUtil.isNotBlank(permJsonDto.getFlowId())) {
                            flowId = permJsonDto.getFlowId();
                            break;
                        }
                    }
                }
            }
        }
        if (StringUtil.isNotBlank(flowId)) {
            //绑定流程，发送http请求
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("busId", task.getId());
            //表单结构
            paramMap.put("busJson", new String(Base64.getEncoder().encode(formInfo.getColumnData().getBytes())));
            //数据
            paramMap.put("dataJson", new String(Base64.getEncoder().encode(param.getDataJson().getBytes())));
            paramMap.put("flowId", flowId);
            paramMap.put("isTable", 1);
            paramMap.put("creatorId", getUser().getUserId());
            paramMap.put("creatorName", getUser().getUserName());
            paramMap.put("operateType", OperateType.download);

            //获取header
            Map<String, String> headers = new HashMap<>();
            headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
            headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
            String resBody = HttpUtil.createPost("https://demo.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
            JSONObject result = JSONObject.parseObject(resBody);
            if (result.getString("code").equals("200")) {
                task.setApproveStatus(1);//1-未审批
                dataExportTaskService.save(task);
                return R.success("正在等待审批，请稍等!");
            } else {
                return R.fail("流程创建失败!");
            }
        } else {//不用走流程，直接导出
            task.setApproveStatus(0);//0-不需审批
            boolean flag = dataExportTaskService.save(task);
            exportService.excute(task);
            return R.status(flag);
        }
    }

    /**
     * @return
     */
    @PostMapping("/doExport")
    @ApiOperation(value = "执行导出操作")
    public R<Object> doExport(String busId, String handleStatus) {
        try {
            BladeUser user = getUser();
            DataExportTask task = dataExportTaskService.getById(busId);
            task.setTenantId(user.getTenantId());
            task.setApproveStatus(2);//2-审批通过
            exportService.excute(task);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return R.success("执行导出操作成功");
    }


}
