package org.springblade.db.cube.service;

import java.util.HashMap;
import java.util.List;

import org.springblade.bi.client.CubeDataClient;
import org.springblade.bi.client.CubeTableColumnClient;
import org.springblade.bi.client.DataSourceClient;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.CubeTableColumnModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.model.CubeTableColumn;
import org.springblade.db.cube.model.CubeTableName;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.TableLinkJoin;
import org.springblade.db.cube.model.TableLinkJoin.JoinKey;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springblade.model.po.VisualdevEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 数据集操作
 * 
 * @author william
 * @date 2022年8月19日 下午2:46:01
 */
@Slf4j
@Service
public class DataSourceService {

	@Autowired
	private CubeDataClient cubeDataClient;

	@Autowired
	private DataSourceClient dataSourceClient;

	@Autowired
	private CubeTableColumnClient tableColumnClient;

	@Autowired
	private CubeTableColumnService cubeTableColumnService;

	public Kv fieldDefault(VisualdevEntity formInfo) {
		Kv kv = Kv.create();
		JSONObject obj = JSON.parseObject(formInfo.getFormData());
		JSONArray array = obj.getJSONArray("fields");
		for (Object object : array) {
			JSONObject json = (JSONObject) object;
			String fieldId = String.valueOf(json.getIntValue("__vModel__"));
			String field = json.getString("__vModelName__");

			JSONObject config = json.getJSONObject("__config__");
			String dataType = config.getString("viewDataType");
			String defaultValue = config.getString("defaultValue");
			String jnpfKey = config.getString("jnpfKey");

			Kv temp = Kv.create();
			temp.set("field", field);
			temp.set("dataType", dataType);
			temp.set("defaultValue", defaultValue);
			temp.set("jnpfKey", jnpfKey);
			kv.set(fieldId, temp);
		}
		return kv;
	}

	/**
	 * 查询数据集的字段的可选性
	 * 
	 * @param cubeId
	 */
	public Kv fieldOption(String cubeId) {
		CubeDataModel cubeData = getCubeData(cubeId);
		log.info("-------cubeId----:"+cubeId);
		List<CubeTableName> cubeTableNames = JSON.parseArray(cubeData.getCubeTableNames(),
				CubeTableName.class);
		DataSourceModel ds = getReportDatasource(cubeId);


		boolean hasLinkTable = RelationUtil.hasLinkTable(cubeData);
		if (!hasLinkTable) {
			// 单表
			List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
					.eq(CubeTableColumn::getCubeDataId, cubeId)
					.orderByAsc(CubeTableColumn::getId)
			);
			return fieldOptionForSingleTable(ds, cubeTableNames.get(0).getTableName(), columnList);
		} else {
			// 多表
			Kv kv = Kv.create();
			String linksJoin = cubeData.getLinksJoin();
			List<TableLinkJoin> joinList = JSON.parseArray(linksJoin, TableLinkJoin.class);
			for (CubeTableName cubeTable : cubeTableNames) {
				List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
						.eq(CubeTableColumn::getCubeDataId, cubeId)
						.eq(CubeTableColumn::getTableName, cubeTable.getTableName())
						.orderByAsc(CubeTableColumn::getId)
				);
				Kv temp = fieldOptionForMultiTable(ds, cubeTable.getTableName(), columnList, joinList);
				if (CollUtil.isNotEmpty(temp)) {
					kv.setAll(temp);
				}
			}
			return kv;
		}
	}

	/**
	 * 是否为被关联字段
	 * 
	 * @param joins
	 * @param table
	 * @param columnName
	 * @return
	 */
	private boolean isJoinedColumn(List<TableLinkJoin> joinList, String table, String columnName) {
		boolean flag = false;
		for (TableLinkJoin join : joinList) {
			if (join.getSource().equalsIgnoreCase(table)) {
				List<JoinKey> joinKeys = join.getJoinKeys();
				for (JoinKey joinKey : joinKeys) {
					if (joinKey.getSourceKey().equalsIgnoreCase(columnName)) {
						flag = true;
					}
				}
			}
		}
		return flag;
	}

	/**
	 * 是否为关联字段，数据来源于父表
	 * 
	 * @param joins
	 * @param table
	 * @param columnName
	 * @return
	 */
	private boolean isJoinColumn(List<TableLinkJoin> joinList, String table, String columnName) {
		boolean flag = false;
		for (TableLinkJoin join : joinList) {
			if (join.getTarget().equalsIgnoreCase(table)) {
				List<JoinKey> joinKeys = join.getJoinKeys();
				for (JoinKey joinKey : joinKeys) {
					if (joinKey.getTargetKey().equalsIgnoreCase(columnName)) {
						flag = true;
					}
				}
			}
		}
		return flag;
	}

	/**
	 * 多单表字段可选性
	 * 
	 * @param ds
	 * @param table
	 * @param cubeColumnList
	 * @param joinList
	 * @return
	 */
	private Kv fieldOptionForMultiTable(DataSourceModel ds, String table, List<CubeTableColumn> cubeColumnList,
			List<TableLinkJoin> joinList) {
		Kv kv = Kv.create();
		IFly fly = IFly.get(ds);
		List<Column> columns = fly.getColumnList(table);
		// log.info("fieldOptionForMultiTable columns = " + columns);
		cubeColumnList.forEach(colCube -> {
			String columnId = String.valueOf(colCube.getId());
			String columnName = colCube.getColumnName();
			Column column = find(columnName, columns);
			if (column != null) {
				Kv temp = Kv.create();
				temp.set("required", column.isRequired());
				temp.set("primaryKey", column.isPrimaryKey());
				temp.set("size", column.getSize());
				temp.set("scale", column.getScale());
				temp.set("column", columnName);
				temp.set("isJoined", isJoinedColumn(joinList, table, columnName));
				temp.set("isJoin", isJoinColumn(joinList, table, columnName));
				kv.set(columnId, temp);
			}
		});
		return kv;
	}

	/**
	 * 单表字段可选性
	 * 
	 * @param ds
	 * @param table
	 * @param cubeColumnList
	 * @return
	 */
	private Kv fieldOptionForSingleTable(DataSourceModel ds, String table, List<CubeTableColumn> cubeColumnList) {
		Kv kv = Kv.create();
		IFly fly = IFly.get(ds);
		if (fly != null){
			List<Column> columns = fly.getColumnList(table);
			// log.info("fieldOptionForSingleTable columns = " + columns);
			cubeColumnList.forEach(colCube -> {
				String columnId = String.valueOf(colCube.getId());
				String columnName = colCube.getColumnName();
				Column column = find(columnName, columns);

				if (column != null) {
					Kv temp = Kv.create();
					temp.set("required", column.isRequired());
					temp.set("primaryKey", column.isPrimaryKey());
					temp.set("size", column.getSize());
					temp.set("scale", column.getScale());
					temp.set("column", columnName);
					temp.set("isJoined", false);
					temp.set("isJoin", false);
					kv.set(columnId, temp);
				}
			});
		}else {
			log.info("---------fly is null");
		}

		return kv;
	}

	private Column find(String columnName, List<Column> columns) {
		for (int i = 0; i < columns.size(); i++) {
			Column column = columns.get(i);
			if (column.getColumnName().equalsIgnoreCase(columnName)) {
				return column;
			}
		}
		return null;
	}

	/**
	 * 获取某个表单对应的主表的主键字段在数据集中的ID
	 * 
	 * @return
	 */
	public Integer getPrimaryKeyCubeIdOfMainTable(VisualdevEntity formInfo) {
		DataSourceModel ds = getReportDatasource(formInfo.getDbLinkId());
		Integer primaryKeyId = null;
		List<RelationBean> relationTree = getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		RelationBean parentRf = relationTree.get(0);
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(parentRf.getTable());
		if (primaryColumn != null) {
			String primaryKey = primaryColumn.getColumnName();
			CubeTableColumn cubeTableColumn = cubeTableColumnService.getByTableAndColumn(parentRf.getTable(),
					primaryKey,Integer.valueOf(formInfo.getDbLinkId()));
			primaryKeyId = cubeTableColumn != null ? cubeTableColumn.getId() : null;
		}
		return primaryKeyId;
	}

	/**
	 * 根据表单提交的字段和表信息 组装表关联的树形结构
	 * 
	 * @param tablesJson 表单提交的字段和表信息
	 * @param cubeId     数据集ID
	 * @return
	 */
	public List<RelationBean> getRelationTree(String tablesJson, String cubeId) {
		CubeDataModel cubeData = cubeDataClient.getById(Integer.decode(cubeId));
		List<RelationBean> list = RelationUtil.rebuild(tablesJson, cubeData);
		List<RelationBean> tree = RelationUtil.rebuildTreeRalation(list);
		return tree;
	}

	/**
	 * 递归入库时向下传递子表需要的关联关系 和 关联字段对应的值
	 * 
	 * @param cubeData    数据集
	 * @param parentTable 父表名
	 * @param parentData  父表入库的数据
	 * @return
	 */
	public HashMap<String, HashMap<String, Object>> getParentTableRelationData(CubeDataModel cubeData,
			String parentTable, HashMap<String, Object> parentData) {
		List<TableLinkJoin> joinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
		// 子表名称，子表字段，子表对应的要入库的父表关联字段的值
		// HashMap<ChildTableName,HashMap<ChildFieldName,SourceData>>
		HashMap<String, HashMap<String, Object>> map = new HashMap<>();
		if (CollUtil.isEmpty(joinList) || CollUtil.isEmpty(parentData)) {
			// 不存在关联关系，直接返回
			log.info("no join relation");
			return map;
		}

		for (TableLinkJoin join : joinList) {
			if (join.getSource().equalsIgnoreCase(parentTable)) {
				// 存在关联关系
				String childTable = join.getTarget();
				HashMap<String, Object> childMap = new HashMap<>();

				List<JoinKey> joinKeys = join.getJoinKeys();
				if (CollUtil.isEmpty(joinKeys)) {
					continue;
				}

				for (JoinKey key : joinKeys) {
					if (key.isTargetKeyVtState() || key.isSourceKeyVtState()) {
						// 虚拟字段，忽略
						continue;
					} else {
						String parentField = key.getSourceKey();
						String childField = key.getTargetKey();
						if (parentData != null && parentData.containsKey(parentField)) {
							Object sourceData = parentData.get(parentField);
							childMap.put(childField, sourceData);
						}
					}
				}
				map.put(childTable, childMap);
			}
		}
		return map;
	}

	/**
	 * 根据数据集ID
	 * 
	 * @param cubeId 数据集ID
	 * @return
	 */
	public List<TableLinkJoin> getTableLinkJoinList(String cubeId) {
		CubeDataModel cubeData = cubeDataClient.getById(Integer.decode(cubeId));
		List<TableLinkJoin> joinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);
		return joinList;
	}

	/**
	 * 查询数据源连接信息
	 * 
	 * @param cubeId 数据集ID
	 * @return
	 */
	public DataSourceModel getReportDatasource(String cubeId) {
		CubeDataModel cubeData = cubeDataClient.getById(Integer.decode(cubeId));
		if (cubeData != null) {
			return dataSourceClient.getById(cubeData.getDatasourceId());
		}
		return null;
	}

	/**
	 * 判断某个数据集是否有关联表
	 * 
	 * @param cubeId
	 * @return
	 */
	public boolean hasLinkTable(String cubeId) {
		CubeDataModel cubeData = getCubeData(cubeId);
		if (cubeData != null) {
			return RelationUtil.hasLinkTable(cubeData);
		}
		return false;
	}

	/**
	 * 查询数据集信息
	 * 
	 * @param cubeId 数据集ID
	 * @return
	 */
	public CubeDataModel getCubeData(String cubeId) {
		CubeDataModel cubeData = null;
		try {
			cubeData = cubeDataClient.getById(Integer.decode(cubeId));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return cubeData;
	}

	/**
	 * 根据字段ID获取字段信息对象
	 */
	public CubeTableColumnModel getFieldInfo(int id) {
		return tableColumnClient.getById(id);
	}

	/**
	 * 根据字段ID获取字段真实名称，例如：COL_2
	 * 
	 * @param id
	 * @return
	 */
	public String getField(int id) {
		CubeTableColumnModel column = getFieldInfo(id);
		if (column != null) {
			return column.getColumnName();
		}
		return null;
	}
}
