package org.springblade.db.database;

import java.io.File;
import java.sql.SQLException;
import java.util.LinkedHashMap;
import java.util.List;

import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.bi.client.model.PageModel;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springblade.model.dto.PaginationModel;
import org.springblade.model.dto.VisualdevDataDto;
import org.springblade.model.po.VisualdevEntity;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;

/**
 * 表单数据操作抽象类
 * 
 * @author William
 * @date 2023年1月6日 下午3:10:16
 */
public abstract class DbBase {
	public abstract R<Object> insert(String modelId, String jsonData) throws SQLException;

	public abstract void recovery(DataSourceModel ds, String table, JSONArray list);

	public abstract R<Kv> batchInsert(VisualdevEntity formInfo, File excelFile, List<ExcelFieldIndex> indexList,
			boolean withHeader) throws SQLException;

	public abstract void truncate(CubeDataModel cubeData, VisualdevEntity formInfo);

	public abstract R<String> delete(String modelId, List<String> idList);

	/**
	 * 根据主键区间值删除数据
	 * 
	 * @param modelId      表单ID
	 * @param keyStart     开始主键
	 * @param keyEnd       结束主键
	 * @param hasLinkTable 是否有关联表
	 * @return
	 */
	public abstract R<String> delete(String modelId, String keyStart, String keyEnd, boolean hasLinkTable);

	public abstract R<String> update(String modelId, String dataId, String jsonData);
	public abstract R<String> batchUpdate(String modelId, List<VisualdevDataDto> formDataList);

	public abstract LinkedHashMap<String, List<JSONObject>> selectForRecycle(String modelId, String dataId);

	public abstract PageModel<JSONObject> page(String modelId, PaginationModel paginationModel);

	/**
	 * 获取某个表的主键字段
	 * 
	 * @param link
	 * @param tableName
	 * @return
	 */
	public String getKey(DataSourceModel ds, String tableName) {
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(tableName);
		return primaryColumn != null ? primaryColumn.getColumnName() : null;
	}
}
