package org.springblade.db.database;

import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.bi.client.model.PageModel;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.enums.DsType;
import org.springblade.db.cube.enums.ViewColumnTypeEnum;
import org.springblade.db.cube.model.CubeTableColumn;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.service.CubeTableColumnService;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.database.action.DeleteAction;
import org.springblade.db.database.action.ImportAction;
import org.springblade.db.database.action.InsertAction;
import org.springblade.db.database.action.SelectAction;
import org.springblade.db.database.action.UpdateAction;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.db.model.UpdateBean;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springblade.model.dto.PaginationModel;
import org.springblade.model.dto.VisualdevDataDto;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.VisualdevService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 表单数据操作通用类
 * 
 * @author William
 * @date 2022年12月30日 上午11:09:35
 */
@Slf4j
@Service
public class DbCommon extends DbBase {

	@Autowired
	private DataSourceService dataSourceService;

	@Autowired
	private DynamicDataService dynamicDataService;

	@Autowired
	private VisualdevService visualdevService;

	@Autowired
	private InsertAction insertAction;

	@Autowired
	private ImportAction importAction;

	@Autowired
	private UpdateAction updateAction;

	@Autowired
	private SelectAction selectAction;

	@Autowired
	private DeleteAction deleteAction;

	@Autowired
	private CubeTableColumnService cubeTableColumnService;

	@Override
	public R<Object> insert(String modelId, String jsonData) throws SQLException{
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		if (formInfo == null) {
			return R.fail("表单不存在");
		}

		if (StrUtil.isBlank(formInfo.getDbLinkId())) {
			return R.fail("缺少数据集ID");
		}

		CubeDataModel cubeData = dataSourceService.getCubeData(formInfo.getDbLinkId());
		DataSourceModel ds = dataSourceService.getReportDatasource(cubeData.getId() + "");
		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		Map<String, Object> formDataMap = JSON.parseObject(jsonData);
		Kv fieldDefault = dataSourceService.fieldDefault(formInfo);
		StringBuffer mainTablePrimaryKeyValue = new StringBuffer();
		insertAction.recursionInsertData(cubeData,
				ds,
				relationTree,
				formDataMap,
				null,
				fieldDefault,
				mainTablePrimaryKeyValue);

		String table = relationTree.get(0).getTable();
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(table);
		boolean hasPrimaryKey = primaryColumn != null;
		if (hasPrimaryKey) {
			Integer primaryKeyCubeIdOfMainTable = dataSourceService.getPrimaryKeyCubeIdOfMainTable(formInfo);
			formDataMap.put(String.valueOf(primaryKeyCubeIdOfMainTable), mainTablePrimaryKeyValue);
		}

		List<CubeTableColumn> columnList = cubeTableColumnService.getListByTableAndCubeId(table,
				Integer.parseInt(formInfo.getDbLinkId()));
		for (CubeTableColumn cubeTableColumn : columnList) {
			if (cubeTableColumn.getViewDataType() == ViewColumnTypeEnum.DATETIME.getValue()) {
				// 数据为日期
				Object data = formDataMap.get(String.valueOf(cubeTableColumn.getId()));
				if (data != null && StrUtil.isNotBlank(data.toString())) {
					long timestamp = Long.parseLong(data.toString());
					String date = DateUtil.formatDateTime(new Date(timestamp));
					formDataMap.put(String.valueOf(cubeTableColumn.getId()), date);
				}
				break;
			}
		}

		return R.data(formDataMap);
	}

	@Override
	public R<Kv> batchInsert(VisualdevEntity formInfo, File excelFile, List<ExcelFieldIndex> indexList,
			boolean withHeader) throws SQLException {
		CubeDataModel cubeData = dataSourceService.getCubeData(formInfo.getDbLinkId());
		boolean isMultipleTable = RelationUtil.hasLinkTable(cubeData);
		R<Kv> ret = null;
//		if (isMultipleTable) {
//			ret = importAction.importForMultipleTable(this, cubeData, withHeader, formInfo, indexList, excelFile);
//		} else {
//			ret = importAction.importForSingleTable(this, cubeData, withHeader, formInfo, indexList, excelFile);
//		}
		ret = importAction.importForSingleTable(this, cubeData, withHeader, formInfo, indexList, excelFile);
		return ret;
	}

	@Override
	public void recovery(DataSourceModel ds, String table, JSONArray list) {
		insertAction.recovery(ds, table, list);
	}

	@Transactional
	@Override
	public void truncate(CubeDataModel cubeData, VisualdevEntity formInfo) {
		try {
			String tableJson = formInfo.getTables();
			List<RelationField> dataList = JSON.parseArray(tableJson, RelationField.class);
			if (CollUtil.isNotEmpty(dataList)) {
				Set<String> tableList = new HashSet<>();
				dataList.forEach(item -> {
					tableList.add(item.getTableName());
				});

				if (CollUtil.isNotEmpty(tableList)) {
					DataSourceModel ds = dataSourceService.getReportDatasource(String.valueOf(cubeData.getId()));
					tableList.forEach(table -> {
						String sql = "TRUNCATE TABLE " + table;
						dynamicDataService.executeSql(ds, sql);
					});
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public R<String> delete(String modelId, String keyStart, String keyEnd, boolean hasLinkTable) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		DataSourceModel ds = dataSourceService.getReportDatasource(formInfo.getDbLinkId());
		if (ds == null) {
			return R.fail("数据源不存在");
		}

		DsType dbType = DsType.reslove(ds.getDsType());
		if (dbType == DsType.Clickhouse) {
			return R.fail("暂不支持Clickhouse数据删除");
		}

		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		String mainTable = relationTree.get(0).getTable();
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(mainTable);
		if (primaryColumn == null) {
			return R.fail("删除失败，该表不存在主键，表名是：" + mainTable);
		}

		// 单表删除
		String primaryKey = primaryColumn.getColumnName();
		String placeHolder = primaryColumn.getJavaType().equalsIgnoreCase("String") ? "'" : "";
		StringBuffer sqlBuffer = new StringBuffer();
		sqlBuffer.append("delete from " + mainTable);
		sqlBuffer.append(" where " + primaryKey + ">=" + placeHolder + keyStart + placeHolder);
		sqlBuffer.append(" and " + primaryKey + "<=" + placeHolder + keyEnd + placeHolder);
		String sql = sqlBuffer.toString();
		log.info("delete sql is " + sql);
		fly.excuteUpdateSql(sql);

//		if (hasLinkTable) {
//			// 关联表批量删除
//			String primaryKey = primaryColumn.getColumnName();
//			String placeHolder = primaryColumn.getJavaType().equalsIgnoreCase("String") ? "'" : "";
//			StringBuffer sqlBuffer = new StringBuffer();
//			sqlBuffer.append("select " + primaryKey + " from " + mainTable);
//			sqlBuffer.append(" where " + primaryKey + ">=" + placeHolder + keyStart + placeHolder);
//			sqlBuffer.append(" and " + primaryKey + "<=" + placeHolder + keyEnd + placeHolder);
//			String sql = sqlBuffer.toString();
//			log.info("select ids sql is " + sql);
//			List<JSONObject> allData = dynamicDataService.selectList(ds, sql);
//			if (CollUtil.isNotEmpty(allData)) {
//				List<String> ids = new ArrayList<>();
//				allData.forEach(data -> {
//					String idKey = data.getString(primaryKey);
//					ids.add(idKey);
//				});
//
//				if (CollUtil.isNotEmpty(ids)) {
//					return deleteAction.deleteMultiTable(ds, relationTree, ids, primaryColumn, relationTree.get(0));
//				}
//			}
//		} else {
//			// 单表删除
//			String primaryKey = primaryColumn.getColumnName();
//			String placeHolder = primaryColumn.getJavaType().equalsIgnoreCase("String") ? "'" : "";
//			StringBuffer sqlBuffer = new StringBuffer();
//			sqlBuffer.append("delete from " + mainTable);
//			sqlBuffer.append(" where " + primaryKey + ">=" + placeHolder + keyStart + placeHolder);
//			sqlBuffer.append(" and " + primaryKey + "<=" + placeHolder + keyEnd + placeHolder);
//			String sql = sqlBuffer.toString();
//			log.info("delete sql is " + sql);
//			fly.excuteUpdateSql(sql);
//		}
		return R.success("操作成功");
	}

	@Override
	public R<String> delete(String modelId, List<String> idList) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		String cubeId = formInfo.getDbLinkId();
		CubeDataModel cubeData = dataSourceService.getCubeData(cubeId);
		DataSourceModel ds = dataSourceService.getReportDatasource(cubeId);
		DsType dbType = DsType.reslove(ds.getDsType());
		if (dbType == DsType.Clickhouse) {
			return R.fail("暂不支持Clickhouse数据删除");
		}

		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		String mainTable = relationTree.get(0).getTable();
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(mainTable);
		if (primaryColumn == null) {
			return R.fail("删除失败，该表不存在主键，表名是：" + mainTable);
		}
		return deleteAction.deleteSingleTable(mainTable, ds, idList);
//		boolean hasLinkTable = RelationUtil.hasLinkTable(cubeData);
//		if (!hasLinkTable) {
//			// 单表删除
//			return deleteAction.deleteSingleTable(mainTable, ds, idList);
//		}
//
//		// 多表删除
//		return deleteAction.deleteMultiTable(ds, relationTree, idList, primaryColumn, relationTree.get(0));
	}

	@Override
	public R<String> update(String modelId, String dataId, String jsonData) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		String cubeId = formInfo.getDbLinkId();
		DataSourceModel ds = dataSourceService.getReportDatasource(cubeId);
		log.info("ds is : " + ds);
		DsType dbType = DsType.reslove(ds.getDsType());
		if (dbType == DsType.Clickhouse) {
			return R.fail("暂不支持Clickhouse数据更新");
		}

		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		String mainTable = relationTree.get(0).getTable();
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(mainTable);
		if (primaryColumn == null) {
			return R.fail("更新失败，该表不存在主键，表名是：" + mainTable);
		}

		Map<String, Object> formDataMap = JSON.parseObject(jsonData);
		List<UpdateBean> updateList = new ArrayList<>();
		DsType dsType = DsType.reslove(ds.getDsType());
		Kv fieldDefault = dataSourceService.fieldDefault(formInfo);
		updateAction.buildUpdate(ds, dsType, dataId, formDataMap, null, null, relationTree, updateList, fieldDefault);
		updateList.forEach(up -> {
			String sql = up.getUpdateSql();
			log.info("update sql is : " + sql);
			dynamicDataService.executeSql(ds, sql);
		});
		return R.success("更新成功");
	}
	@Override
	public R<String> batchUpdate(String modelId, List<VisualdevDataDto> formDataList) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		String cubeId = formInfo.getDbLinkId();
		DataSourceModel ds = dataSourceService.getReportDatasource(cubeId);
		DsType dbType = DsType.reslove(ds.getDsType());
		if (dbType == DsType.Clickhouse) {
			return R.fail("暂不支持Clickhouse数据更新");
		}

		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		String mainTable = relationTree.get(0).getTable();
		IFly fly = IFly.get(ds);
		Column primaryColumn = fly.getPrimaryColumn(mainTable);
		if (primaryColumn == null) {
			return R.fail("更新失败，该表不存在主键，表名是：" + mainTable);
		}
		DsType dsType = DsType.reslove(ds.getDsType());
		Kv fieldDefault = dataSourceService.fieldDefault(formInfo);
		if (formDataList != null && formDataList.size()>0){
			formDataList.forEach(formData->{
				Map<String, Object> formDataMap = JSON.parseObject(formData.getData()+"");
				List<UpdateBean> updateList = new ArrayList<>();
				updateAction.buildUpdate(ds, dsType, formData.getId(), formDataMap, null, null, relationTree, updateList, fieldDefault);
				updateList.forEach(up -> {
					String sql = up.getUpdateSql();
					log.info("update sql is : " + sql);
					dynamicDataService.executeSql(ds, sql);
				});
			});
		}
		return R.success("更新成功");
	}

	@Override
	public LinkedHashMap<String, List<JSONObject>> selectForRecycle(String modelId, String dataId) {
		return selectAction.selectForRecycle(modelId, dataId);
	}

	@Override
	public PageModel<JSONObject> page(String modelId, PaginationModel paginationModel) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		DataSourceModel ds = dataSourceService.getReportDatasource(formInfo.getDbLinkId());
		if (ds == null) {
			return new PageModel<JSONObject>();
		}

		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		String sql = selectAction.buildListSql(relationTree, ds);
		log.info("list sql is " + sql);
		CubeDataModel cubeData = dataSourceService.getCubeData(formInfo.getDbLinkId());
		DataSourceModel dataSource = dataSourceService.getReportDatasource(String.valueOf(cubeData.getId()));
		Page<JSONObject> page = new Page<JSONObject>(paginationModel.getCurrentPage(), paginationModel.getPageSize())
				.setSearchCount(true);
		PageModel<JSONObject> pageData = dynamicDataService.selectPage(page, dataSource, sql);
		return pageData;
	}

}
