package org.springblade.db.database;

import java.io.File;
import java.sql.SQLException;
import java.util.LinkedHashMap;
import java.util.List;

import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.bi.client.model.PageModel;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.model.dto.PaginationModel;
import org.springblade.model.dto.VisualdevDataDto;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.VisualdevService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;

/**
 * 表单数据操作
 * 
 * @author william
 * @date 2022年8月16日 上午11:05:13
 */
@Service
public class FormDataService {
	@Autowired
	private DataSourceService dataSourceService;
	
	@Autowired
	private VisualdevService visualdevService;

	@Autowired
	private DbCommon dbCommon;
	
	public DataSourceModel getDs(String modelId) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		if (formInfo == null) {
			return null;
		}
		return dataSourceService.getReportDatasource(formInfo.getDbLinkId());
	}

	/**
	 * 单条入库
	 * 
	 * @param modelId  表单ID
	 * @param jsonData 前端提交的数据参数
	 * @return
	 */
	public R<Object> add(String modelId, String jsonData) throws SQLException{
		return dbCommon.insert(modelId, jsonData);
	}

	/**
	 * 批量导入
	 * 
	 * @param cubeData
	 * @param withHeader
	 * @param formInfo
	 * @param indexList
	 * @param excelFile
	 * @param isMultipleTable
	 * @return
	 * @throws SQLException
	 */
	public R<Kv> batchAdd(VisualdevEntity formInfo, File excelFile, List<ExcelFieldIndex> indexList,
			boolean withHeader)
			throws SQLException {
		long startTime = System.currentTimeMillis();
		R<Kv> ret = dbCommon.batchInsert(formInfo, excelFile, indexList, withHeader);
		long endTime = System.currentTimeMillis();
		float takeUpTime = (endTime - startTime) / 1000.f;
		ret.getData().set("takeUpTime", takeUpTime);
		return ret;
	}
	
	public void recovery(DataSourceModel ds,String table, JSONArray list) {
		dbCommon.recovery(ds,table, list);
	}
	
	/**
	 * 删除表单数据
	 * 
	 * @param formInfo
	 * @param idList
	 * @return
	 */
	public R<String> delete(String modelId, List<String> idList) {
		return dbCommon.delete(modelId, idList);
	}
	
	public R<String> delete(String modelId, String keyStart, String keyEnd, boolean hasLinkTable){
		return dbCommon.delete(modelId, keyStart, keyEnd, hasLinkTable);
	}

	/**
	 * 更新数据
	 * 
	 * @param formInfo 表单信息
	 * @param id       数据主键
	 * @param jsonData 新数据
	 * @return
	 */
	public R<String> update(String modelId, String dataId, String jsonData) {
		return dbCommon.update(modelId, dataId, jsonData);
	}
	/**
	 * 更新数据
	 *
	 * @return
	 */
	public R<String> batchUpdate(String modelId, List<VisualdevDataDto> formDataList) {
		return dbCommon.batchUpdate(modelId, formDataList);
	}

	/**
	 * 分页查询 仅返回分页数据
	 * 
	 * @param formInfo
	 * @param paginationModel
	 * @return
	 */
	public PageModel<JSONObject> page(String modelId, PaginationModel paginationModel) {
		return dbCommon.page(modelId, paginationModel);
	}
	
	public LinkedHashMap<String, List<JSONObject>> selectForRecycle(String modelId, String dataId) {
		return dbCommon.selectForRecycle(modelId, dataId);
	}

	/**
	 * 清空某个表单对应的数据集相关表
	 * 
	 * @param cubeData
	 * @param formInfo
	 */
	public void truncate(CubeDataModel cubeData, VisualdevEntity formInfo) {
		dbCommon.truncate(cubeData, formInfo);
	}

}
