package org.springblade.db.database.action;

import java.util.List;
import java.util.stream.Collectors;

import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.api.R;
import org.springblade.db.cube.enums.DsType;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationKey;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 
 * @author William
 * @date 2023年1月6日 下午4:46:34
 */
@Slf4j
@Service
public class DeleteAction {
	@Autowired
	private DynamicDataService dynamicDataService;
	
	@Autowired
	private DeleteActionOracle oracleDelete;
	
	/**
	 * 单表批量删除
	 * 
	 * @param formInfo
	 * @param idList
	 * @return
	 */
	public R<String> deleteSingleTable(String tableName, DataSourceModel ds, List<String> idList) {
		try {
			IFly fly = IFly.get(ds);
			Column primaryColumn = fly.getPrimaryColumn(tableName);
			StringBuffer sqlBuffer = new StringBuffer();
			String idsStr = idList.stream().collect(Collectors.joining(","));
			if (primaryColumn.getJavaType().equalsIgnoreCase("String")) {
				// 主键为字符串类型
				idsStr = idList.stream().collect(Collectors.joining("', '"));
				idsStr = "'" + idsStr + "'";
			}
			sqlBuffer.append("delete from " + tableName + " ");
			sqlBuffer.append("where " + primaryColumn.getColumnName() + " ");
			sqlBuffer.append("in (" + idsStr + ")");
			String sql = sqlBuffer.toString();
			log.info("delete sql is : " + sql);
			dynamicDataService.executeSql(ds, sql);
			return R.success("删除成功");
		} catch (Exception e) {
		}
		return R.fail("删除失败");
	}
	
	/**
	 * 多表删除数据，支持批量操作
	 * 
	 * @param ds
	 * @param relationTree
	 * @param idList
	 * @param primaryColumn
	 * @param mainTableBean
	 * @return
	 */
	public R<String> deleteMultiTable(DataSourceModel ds, List<RelationBean> relationTree,
			List<String> idList, Column primaryColumn, RelationBean mainTableBean) {
		DsType dbType = DsType.reslove(ds.getDsType());
		if (dbType == DsType.MySQL) {
			return deleteMultiTableForMysql(ds, relationTree, idList, primaryColumn, mainTableBean);
		}
		return oracleDelete.deleteMultiTable(ds, relationTree, idList);
	}
	
	/**
	 * MYSQL多表删除数据
	 * 
	 * @param tableName
	 * @param ds
	 * @param idList
	 * @return
	 */
	private R<String> deleteMultiTableForMysql(DataSourceModel ds, List<RelationBean> relationTree,
			List<String> idList, Column primaryColumn, RelationBean mainTableBean) {
		try {
			StringBuffer sqlBuffer = new StringBuffer();
			sqlBuffer.append("delete " + mainTableBean.getTableName() + ",");
			StringBuffer afterDelete = new StringBuffer();
			StringBuffer joinBuffer = new StringBuffer();
			buildDeleteStringBuffer(mainTableBean.getTableName(), relationTree, afterDelete, joinBuffer);

			String deleteSql = afterDelete.toString();
			sqlBuffer.append(deleteSql.substring(0, deleteSql.length() - 1));

			StringBuffer fromBuffer = new StringBuffer();
			fromBuffer.append(" from " + mainTableBean.getTable());
			fromBuffer.append(" " + mainTableBean.getTableName() + " ");

			sqlBuffer.append(fromBuffer);
			sqlBuffer.append(joinBuffer);

			StringBuffer whereBuffer = new StringBuffer();
			whereBuffer.append(" where " + mainTableBean.getTableName() + "." + primaryColumn.getColumnName());
			String idsStr = idList.stream().collect(Collectors.joining(","));
			if (primaryColumn.getJavaType().equalsIgnoreCase("String")) {
				// 主键为字符串类型
				idsStr = idList.stream().collect(Collectors.joining("', '"));
				idsStr = "'" + idsStr + "'";
			}
			whereBuffer.append(" in (" + idsStr + ")");
			sqlBuffer.append(whereBuffer);
			String sql = sqlBuffer.toString();
			log.info("batch delete sql is : " + sql);
			dynamicDataService.executeSql(ds, sql);
			return R.success("删除成功");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return R.fail("删除失败");
	}

	/**
	 * 多表关联删除的方式仅在mysql上测试成功
	 * 
	 * @param parentTableAlias
	 * @param relationTree
	 * @param afterDelete
	 * @param joinBuffer
	 */
	private void buildDeleteStringBuffer(String parentTableAlias, List<RelationBean> relationTree,
			StringBuffer afterDelete, StringBuffer joinBuffer) {

		for (RelationBean relation : relationTree) {
			if (relation.getRelationTable().equalsIgnoreCase(RelationUtil.ROOT_TABLE_PARENT)) {
				// 当前是主表
				buildDeleteStringBuffer(relation.getTableName(), relation.getChildren(), afterDelete, joinBuffer);
				return;
			}
			afterDelete.append(relation.getTableName() + ",");
			joinBuffer.append("left join " + relation.getTable() + " " + relation.getTableName() + " ");

			StringBuffer onBuffer = new StringBuffer();
			onBuffer.append("on ");

			List<RelationKey> relationKeyList = relation.getKeys();
			for (int i = 0, size = relationKeyList.size(); i < size; i++) {
				RelationKey rk = relationKeyList.get(i);
				String sourceKey = rk.getSourceKey();
				String targetKey = rk.getTargetKey();
				if (i != 0) {
					onBuffer.append(" and ");
				}
				onBuffer.append(parentTableAlias + "." + sourceKey);
				onBuffer.append(" = ");
				onBuffer.append(relation.getTableName() + "." + targetKey);
				onBuffer.append(" ");
			}
			joinBuffer.append(onBuffer);

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				buildDeleteStringBuffer(relation.getTableName(), relation.getChildren(), afterDelete, joinBuffer);
			}
		}
	}
}
