package org.springblade.db.database.action;

import java.io.File;
import java.sql.SQLException;
import java.util.List;

import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.database.DbBase;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.db.excel.import_.ExcelRead4MultipleTableListener;
import org.springblade.db.excel.import_.ExcelRead4SingleTableListener;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.ResetSerialnumberService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.excel.EasyExcel;

import javax.annotation.Resource;

/**
 * 
 * @author William
 * @date 2023年1月6日 下午3:58:16
 */
@Service
@Slf4j
public class ImportAction {
	@Autowired
	private DynamicDataService dynamicDataService;

	@Autowired
	private DataSourceService dataSourceService;

	@Resource
	ResetSerialnumberService resetSerialnumberService;

	/**
	 * 多表批量导入数据
	 * 
	 * @param cubeData   数据集信息
	 * @param withHeader 是否带表头
	 * @param formInfo   表单信息
	 * @param indexList  字段和excel索引映射关系
	 * @param excelFile  excel文件
	 * @return
	 * @throws SQLException
	 */
	public R<Kv> importForMultipleTable(DbBase dbBase, CubeDataModel cubeData, boolean withHeader,
			VisualdevEntity formInfo, List<ExcelFieldIndex> indexList, File excelFile) throws SQLException {
		Kv fieldOption = dataSourceService.fieldOption(String.valueOf(cubeData.getId()));
		log.info("多表批量导入数据----------");
		log.info("fieldOption----------"+fieldOption);
		Kv fieldDefault = dataSourceService.fieldDefault(formInfo);
		log.info("fieldDefault----------"+fieldDefault);
		DataSourceModel ds = dataSourceService.getReportDatasource(String.valueOf(cubeData.getId()));
		ExcelRead4MultipleTableListener listener = new ExcelRead4MultipleTableListener(dbBase, dataSourceService,
				cubeData, dynamicDataService, ds, formInfo, indexList, fieldOption, fieldDefault);
		EasyExcel.read(excelFile, listener).sheet().headRowNumber(withHeader ? 1 : 0).doRead();
		Kv kv = Kv.create();
		kv.set("count", listener.getDataTotal());
		kv.set("keyStart", listener.getKeyStart());
		kv.set("keyEnd", listener.getKeyEnd());
		return R.data(kv, "共导入" + listener.getTableTotal() + "张表，" + listener.getDataTotal() + "条数据");
	}

	/**
	 * 单表批量导入数据
	 * 
	 * @param cubeData   数据集信息
	 * @param withHeader 是否带表头
	 * @param formInfo   表单信息
	 * @param indexList  字段和excel索引映射关系
	 * @param excelFile  excel文件
	 * @return
	 * @throws SQLException
	 */
	public R<Kv> importForSingleTable(DbBase dbBase, CubeDataModel cubeData, boolean withHeader,
			VisualdevEntity formInfo, List<ExcelFieldIndex> indexList, File excelFile) throws SQLException {
		Kv fieldOption = dataSourceService.fieldOption(String.valueOf(cubeData.getId()));
		log.info("单表批量导入数据----------");
		log.info("fieldOption----------"+fieldOption);
		Kv fieldDefault = dataSourceService.fieldDefault(formInfo);
		log.info("fieldDefault----------"+fieldDefault);
		DataSourceModel ds = dataSourceService.getReportDatasource(String.valueOf(cubeData.getId()));
		ExcelRead4SingleTableListener listener = new ExcelRead4SingleTableListener(dbBase, dataSourceService, cubeData,
				dynamicDataService, ds, formInfo, indexList, fieldOption, fieldDefault);
		EasyExcel.read(excelFile, listener).sheet().headRowNumber(withHeader ? 1 : 0).doRead();
		Kv kv = Kv.create();
		kv.set("count", listener.getTotal());
		kv.set("keyStart", listener.getKeyStart());
		kv.set("keyEnd", listener.getKeyEnd());
		return R.data(kv, "共导入1张表" + listener.getTotal() + "条数据");
	}
}
