package org.springblade.db.database.action;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.github.xiaoymin.knife4j.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.config.JnpfKeyConsts;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.enums.DsType;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springblade.util.SystemComponent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author William
 * @date 2023年1月6日 下午3:51:39
 */
@Slf4j
@Service
public class InsertAction {

    @Autowired
    private DynamicDataService dynamicDataService;

    @Autowired
    private DataSourceService dataSourceService;

    @Autowired
    private SystemComponent sysCompoment;

    /**
     * 递归插入数据
     *
     * @param cubeData        数据集
     * @param ds              数据库连接对象
     * @param relationTree    表关联及字段信息
     * @param formDataMap     前端提交的所有数据集合
     * @param relationDataMap 父表关联字段数据
     */
    public void recursionInsertData(CubeDataModel cubeData, DataSourceModel ds, List<RelationBean> relationTree,
                                    Map<String, Object> formDataMap, HashMap<String, HashMap<String, Object>> relationDataMap,
                                    Kv fieldDefault, StringBuffer mainTablePrimaryKeyValue) throws SQLException{
        log.info("insertAction.recursionInsertData********************************");
        RelationBean relation = relationTree.get(0);
        String table = relation.getTable();// 表名
        IFly fly = IFly.get(ds);
        Column primaryColumn = fly.getPrimaryColumn(table);
        boolean hasPrimaryKey = primaryColumn != null;
        String tableKey = hasPrimaryKey ? primaryColumn.getColumnName() : null;

        HashMap<String, Object> tableData = new HashMap<>();// 当前表要入库的字段和数据
        List<RelationField> fieldList = relation.getFields();
        StringBuffer sqlBuffer = new StringBuffer("insert into " + table + " ");
        StringBuffer fieldSql = new StringBuffer("(");
        StringBuffer valueSql = new StringBuffer("values(");
        for (int i = 0, size = fieldList.size(); i < size; i++) {
            RelationField field = fieldList.get(i);

            String value = null;
            String fieldId = String.valueOf(field.getFieldId());
            String fieldName = field.getField();
            if (formDataMap.containsKey(fieldId)) {
                // 从前端输入内容取值
                Object data = formDataMap.get(fieldId);
                value = data == null || data.toString().equalsIgnoreCase("[]") ? null : data.toString();
            }

            Kv fieldDefaultTemp = ((Kv) fieldDefault.get(fieldId));
            String jnpfKey = fieldDefaultTemp != null ? fieldDefaultTemp.getStr("jnpfKey") : "";
            if (sysCompoment.isSystemComponent(jnpfKey)) {
                // 系统默认组件，值由系统生成
                if (!jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYTIME)
                        && !jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYUSER)
                        && !jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYUSERID)) {
                    // 字段不是修改相关，则自动生成
                    value = sysCompoment.getSystemValue(jnpfKey).toString();
                }
            }

            // 某个字段是否存在关联字段，如果存在,则从父表的关联字段取值
            if (CollUtil.isNotEmpty(relationDataMap) && relationDataMap.containsKey(table)) {
                HashMap<String, Object> map = relationDataMap.get(table);
                if (CollUtil.isNotEmpty(map) && map.containsKey(fieldName)) {
                    Object tempValue = map.get(fieldName);
                    if (tempValue != null) {
                        // 该字段为关联字段，数据应该来源于父表
                        String valueFromParent = tempValue.toString();
                        value = valueFromParent;
                    }
                }
            }

            if (StrUtil.isBlank(tableKey) || !fieldName.equalsIgnoreCase(tableKey)) {
                // 没有主键或者输入的字段不是主键则拼接，否则如果是主键则预生成
                fieldSql.append(field.getField());
                tableData.put(fieldName, value);

                if (value != null) {
                    String dataType = field.getDataType();
                    if (dataType.equalsIgnoreCase("varchar")) {
                        value = "'" + value + "'";
                    } else if (dataType.equalsIgnoreCase("datetime")) {
                        // 时间日期类型
                        if (StrUtil.isNotBlank(value)) {
                            long timestamp = Long.parseLong(value);
                            value = DateUtil.formatDateTime(new Date(timestamp));
                            value = "'" + value + "'";
                            DsType dsType = DsType.reslove(ds.getDsType());
                            if (dsType == DsType.Oracle) {
                                value = "to_date(" + value + ",'yyyy-mm-dd HH24:mi:ss')";
                            }
                        }
                    }
                }
                valueSql.append(value);

                fieldSql.append(",");
                valueSql.append(",");
            }
        }

        // 需要删除sql语句中 多余的逗号
        String fsql = fieldSql.toString();
        fieldSql = new StringBuffer();
        fieldSql.append(fsql.substring(0, fsql.toString().length() - 1));

        String vsql = valueSql.toString();
        valueSql = new StringBuffer();
        valueSql.append(vsql.toString().substring(0, vsql.toString().length() - 1));

        // 当前表有主键
        if (hasPrimaryKey && !primaryColumn.getJavaType().equalsIgnoreCase("Integer")) {
            // 主键为非自增整型，则预生成主键并入库
            long tableKeyValue = IdWorker.getId();
            fieldSql.append("," + tableKey);
            valueSql.append("," + tableKeyValue);
            tableData.put(tableKey, tableKeyValue);

            if (relation.getRelationTable().equalsIgnoreCase(RelationUtil.ROOT_TABLE_PARENT)) {
                // 当前为主表
                mainTablePrimaryKeyValue.delete(0, mainTablePrimaryKeyValue.length());
                mainTablePrimaryKeyValue.append(tableKeyValue);
            }
        } else {
            // 主键为自增则忽略
        }

        fieldSql.append(") ");
        valueSql.append(") ");

        sqlBuffer.append(fieldSql);
        sqlBuffer.append(valueSql);

        String sql = sqlBuffer.toString();
        log.info("insert sql is : " + sql);
        DataSourceModel dataSource = dataSourceService.getReportDatasource(String.valueOf(cubeData.getId()));
        JSONObject jsonObject = dynamicDataService.executeSql(dataSource, sql);
        if (jsonObject == null || "500".equalsIgnoreCase((String) jsonObject.get("code"))) {
            throw new SQLException("sql执行失败!");
        }
    }

    public void recovery(DataSourceModel ds, String table, JSONArray list) {
        IFly fly = IFly.get(ds);
        List<Column> columnList = fly.getColumnList(table);

        StringBuffer sqlBuffer = new StringBuffer("insert into " + table + " ");
        StringBuffer fieldSql = new StringBuffer("(");
        StringBuffer valueSql = new StringBuffer("values(");
        for (Object object : list) {
            if (object != null) {
                JSONObject json = JSON.parseObject(object.toString());
                if (json != null) {
                    for (int i = 0, size = columnList.size(); i < size; i++) {
                        Column col = columnList.get(i);
                        String javaType = col.getJavaType();

                        String column = col.getColumnName();
                        Object valueObj = json.get(column);
                        if (valueObj != null) {
                            fieldSql.append(column);
                            if (javaType.equalsIgnoreCase("String")) {
                                // 字符串
                                valueSql.append("'" + valueObj.toString() + "'");
                            } else if (javaType.equalsIgnoreCase("Date")) {
                                // 日期
                                DsType dsType = DsType.reslove(ds.getDsType());
                                if (dsType == DsType.Oracle) {
                                    String valueTemp = "to_date('" + valueObj.toString() + "','yyyy-mm-dd HH24:mi:ss')";
                                    valueSql.append(valueTemp);
                                } else {
                                    valueSql.append("'" + valueObj.toString() + "'");
                                }
                            } else if (javaType.equalsIgnoreCase("BigDecimal")) {
                                // 带小数点
                                valueSql.append(valueObj.toString());
                            } else {
                                // 数值
                                valueSql.append(valueObj.toString());
                            }
                            fieldSql.append(",");
                            valueSql.append(",");
                        }
                    }
                }
            }
        }

        fieldSql.deleteCharAt(fieldSql.length() - 1);
        valueSql.deleteCharAt(valueSql.length() - 1);
        fieldSql.append(") ");
        valueSql.append(") ");

        sqlBuffer.append(fieldSql);
        sqlBuffer.append(valueSql);

        String sql = sqlBuffer.toString();
        log.info("recovery sql is : " + sql);
        fly.excuteUpdateSql(sql);
    }
}
