package org.springblade.db.database.action;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.db.cube.enums.JoinTypeEnum;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.model.RelationKey;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.VisualdevService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;

import cn.hutool.core.collection.CollUtil;

/**
 * 
 * @author William
 * @date 2023年1月6日 下午4:33:15
 */
@Service
public class SelectAction {
	@Autowired
	private DataSourceService dataSourceService;
	
	@Autowired
	private VisualdevService visualdevService;
	
	@Autowired
	private DeleteActionOracle dbAction;
	
	/**
	 * 根据主表id查询所有的关联数据
	 * 
	 * @param ds
	 * @param modelId
	 * @param dataId
	 * @return
	 */
	public LinkedHashMap<String, List<JSONObject>> selectForRecycle( String modelId, String dataId) {
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		DataSourceModel ds = dataSourceService.getReportDatasource(formInfo.getDbLinkId());
		List<RelationBean> relationTree = dataSourceService.getRelationTree(formInfo.getTables(),
				formInfo.getDbLinkId());
		List<String> idList = new ArrayList<>();
		idList.add(dataId);
		LinkedHashMap<String, List<JSONObject>> allData = dbAction.selectDataListWillDelete(null,
				ds,
				relationTree,
				idList,
				null);
		return allData;
	}

	/**
	 * 构造完整的查询语句
	 * 
	 * @param relationTree
	 * @return
	 */
	public String buildListSql(List<RelationBean> relationTree, DataSourceModel ds) {
		StringBuffer sqlBuffer = new StringBuffer();
		String fieldSql = buildFieldSql(null, relationTree, ds).toString();
		// 去掉最后一个逗号
		fieldSql = fieldSql.substring(0, fieldSql.length() - 1);
		sqlBuffer.append("select " + fieldSql);
		sqlBuffer.append(" from  " + buildFromSql(null, null, relationTree));
		sqlBuffer.append(" where 1=1 ");
		return sqlBuffer.toString();
	}

	/**
	 * 构造所有字段的查询语句片段
	 * 
	 * @param sqlBuffer
	 * @param relationTree
	 * @return
	 */
	private StringBuffer buildFieldSql(StringBuffer sqlBuffer, List<RelationBean> relationTree, DataSourceModel ds) {
		if (sqlBuffer == null) {
			sqlBuffer = new StringBuffer();
		}
		for (int i = 0, size = relationTree.size(); i < size; i++) {
			RelationBean relation = relationTree.get(i);
			StringBuffer singleSqlBuffer = buildFieldSqlForSingle(relation, ds);
			if (singleSqlBuffer.length() > 0) {
				sqlBuffer.append(singleSqlBuffer);
				sqlBuffer.append(",");
			}

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				// 递归继续拼接
				buildFieldSql(sqlBuffer, relation.getChildren(), ds);
			}
		}
		return sqlBuffer;
	}

	/**
	 * 构造单表字段的查询语句片段
	 * 
	 * @param relation
	 * @return
	 */
	private StringBuffer buildFieldSqlForSingle(RelationBean relation, DataSourceModel ds) {
		StringBuffer sqlBuffer = new StringBuffer();
		List<RelationField> fieldList = relation.getFields();
		for (int i = 0, size = fieldList.size(); i < size; i++) {
			RelationField field = fieldList.get(i);
			sqlBuffer.append(field.getTableAliasName() + "." + field.getField());
			sqlBuffer.append(" as \"" + field.getFieldId() + "\"");

			if (i < size - 1) {
				sqlBuffer.append(",");
			}
		}
		return sqlBuffer;
	}

	/**
	 * 构造from后面的的查询语句片段
	 * 
	 * @param sqlBuffer
	 * @param parent
	 * @param relationTree
	 * @return
	 */
	private StringBuffer buildFromSql(StringBuffer sqlBuffer, RelationBean parent, List<RelationBean> relationTree) {
		if (sqlBuffer == null) {
			sqlBuffer = new StringBuffer();
		}
		for (int i = 0, size = relationTree.size(); i < size; i++) {
			RelationBean relation = relationTree.get(i);
			if (CollUtil.isEmpty(relation.getKeys())) {
				// 当前是主表
				sqlBuffer.append(relation.getTable() + " " + relation.getTableName() + " ");
			} else {
				// 当前是子表
				JoinTypeEnum joinType = JoinTypeEnum.valueOf(relation.getJoinType());
				String joinSql = joinType.getSql();
				sqlBuffer.append(joinSql + relation.getTable() + " " + relation.getTableName() + " on ");

				List<RelationKey> keys = relation.getKeys();
				for (RelationKey key : keys) {
					sqlBuffer.append(parent.getTableName() + "." + key.getSourceKey());
					sqlBuffer.append(" = ");
					sqlBuffer.append(relation.getTableName() + "." + key.getTargetKey());
					sqlBuffer.append(" ");
				}
			}

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				// 递归继续拼接
				buildFromSql(sqlBuffer, relation, relation.getChildren());
			}
		}
		return sqlBuffer;
	}
}
