package org.springblade.db.excel.import_;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.db.cube.enums.ViewColumnTypeEnum;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.model.TableLinkJoin;
import org.springblade.db.cube.model.TableLinkJoin.JoinKey;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.database.DbBase;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.db.model.TableData;
import org.springblade.db.model.TableFieldData;
import org.springblade.db.model.TableRowData;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.util.SystemComponent;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 多表excel导入
 * 
 * @author william
 * @date 2022年9月15日 下午3:39:36
 */
@Slf4j
public class ExcelRead4MultipleTableListener extends AnalysisEventListener<LinkedHashMap<Integer, String>> {
	private static final int BATCH_COUNT = 10000;

	private long dataTotal;
	private long tableTotal;

	private DataSourceModel ds;
	private DynamicDataService dynamicDataService;

	private List<RelationField> fieldList;
	private List<TableLinkJoin> joinList;
	private List<RelationBean> relationTree;
	/**
	 * 主表对象
	 */
	private RelationBean mainTableBean;

	private Map<String, String> keyMap = new HashMap<>();// 每个表对应的主键

	private TableData tableData = new TableData();

	private boolean firstInvoke = true;
	private String keyStart;
	private String keyEnd;

	private Kv fieldOption;
	private Kv fieldDefault;
	private SystemComponent sysCompoment;

	public ExcelRead4MultipleTableListener(DbBase dbBase,
			DataSourceService dataSourceService, CubeDataModel cubeData, DynamicDataService dynamicDataService,
			DataSourceModel ds, VisualdevEntity formInfo, List<ExcelFieldIndex> sourceIndexList, Kv fieldOption,
			Kv fieldDefault) throws SQLException {
		this.ds = ds;
		this.dynamicDataService = dynamicDataService;
		this.joinList = JSON.parseArray(cubeData.getLinksJoin(), TableLinkJoin.class);

		this.fieldOption = fieldOption;
		this.fieldDefault = fieldDefault;
		this.sysCompoment = SpringUtil.getBean(SystemComponent.class);

		// // 按照excel索引升序排列
		// List<ExcelFieldIndex> indexList = sourceIndexList.stream()
		// .sorted(Comparator.comparing(ExcelFieldIndex::getIndex)).collect(Collectors.toList());

		// 按照excel索引升序排列
		// this.fieldList =
		// fieldList.stream().sorted(Comparator.comparing(RelationField::getIndex))
		// .collect(Collectors.toList());

		this.fieldList = new ArrayList<>();
		String tables = formInfo.getTables();
		List<RelationField> sourceFieldList = JSON.parseArray(tables, RelationField.class);
		for (RelationField rf : sourceFieldList) {
			for (ExcelFieldIndex excel : sourceIndexList) {
				if (rf.getFieldId() == excel.getFieldId()) {
					// 给RelationField实体类对象赋值excel下表索引
					rf.setIndex(excel.getIndex());
					this.fieldList.add(rf);
					continue;
				}
			}

			String tableName = rf.getTableName();
			if (!keyMap.containsKey(tableName)) {
				String key = dbBase.getKey(ds, rf.getTableName());
				keyMap.put(tableName, key);
			}

		}

		tables = JSON.toJSONString(sourceFieldList);
		this.relationTree = dataSourceService.getRelationTree(tables, formInfo.getDbLinkId());
		mainTableBean = this.relationTree.get(0);
	}

	/**
	 * 根据excel下表找到对应的RelationField实体类
	 * 
	 * @param excelIndex
	 * @return
	 */
	private RelationField getFieldByIndex(int excelIndex) {
		for (RelationField field : fieldList) {
			if (field.getIndex() == excelIndex) {
				return field;
			}
		}

		return null;
	}

	/**
	 * 生成所有表对应的行数据结构,key是表名，value是行数据
	 * 
	 * @return
	 */
	private Map<String, TableRowData> buildAllTableRow() {
		Map<String, TableRowData> map = new HashMap<>();
		for (RelationField field : fieldList) {
			map.put(field.getTableName(), new TableRowData());
		}
		return map;
	}

	@Override
	public void invoke(LinkedHashMap<Integer, String> row, AnalysisContext context) {
		// key是表名，value是行数据
		Map<String, TableRowData> tableRowMap = buildAllTableRow();
		List<Integer> excelIndexList = new ArrayList<>(row.keySet());
		// 循环组装行原始数据
		for (Integer index : excelIndexList) {
			RelationField rf = getFieldByIndex(index);
			if (rf != null) {
				// 表名
				String table = rf.getTableName();
				// 字段
				String field = rf.getField();
				// 值
				String value = row.get(index);
				// 字段类型
				ViewColumnTypeEnum type = ViewColumnTypeEnum.valueOf(rf.getDataTypeOrigin());

				String fieldId = String.valueOf(rf.getFieldId());
				Kv fieldDefaultTemp = ((Kv) fieldDefault.get(fieldId));
				String jnpfKey = fieldDefaultTemp != null ? fieldDefaultTemp.getStr("jnpfKey") : "";
				if (sysCompoment.isSystemComponent(jnpfKey)) {
					// 系统默认组件，值由系统生成
					value = sysCompoment.getSystemValue(jnpfKey).toString();
				} else {
					String defValue = ((Kv) this.fieldDefault.get(fieldId)).getStr("defaultValue");
					boolean required = ((Kv) this.fieldOption.get(fieldId)).getBool("required");
					boolean primaryKey = ((Kv) this.fieldOption.get(fieldId)).getBool("primaryKey");
					boolean isJoined = ((Kv) this.fieldOption.get(fieldId)).getBool("isJoined");
					if ((required || isJoined) && !primaryKey && StrUtil.isBlank(defValue) && StrUtil.isBlank(value)) {
						// 必填，非主键，无默认值，并且要导入的数据为空,则跳过
						continue;
					}
				}

				TableRowData tableRow = tableRowMap.get(table);
				TableFieldData fieldData = new TableFieldData(type, field, value);
				tableRow.add(field, fieldData);
			}
		}

		// 判断每个表的主键，存在主键的话并赋值，并且加入到行数据
		List<String> tableKeys = new ArrayList<>(tableRowMap.keySet());
		tableKeys.forEach(table -> {
			String key = keyMap.get(table);
			if (StrUtil.isNotBlank(key)) {
				// 如果该表有主键
				String keyValue = IdWorker.getIdStr();
				TableRowData tableRow = tableRowMap.get(table);
				TableFieldData keyData = new TableFieldData(ViewColumnTypeEnum.BIGINT, key, keyValue);
				tableRow.add(key, keyData);

				if (table.equalsIgnoreCase(mainTableBean.getTable())) {
					// 当前为主表，则记录导入数据时的开始主键和结束主键
					if (firstInvoke) {
						keyStart = keyValue;
						firstInvoke = false;
					}
					keyEnd = keyValue;
				}
			}
		});

		// 判断关联字段，如果存在，则给关联子表字段赋值对应的父表相应字段的值,并加入到行数据
		if (CollUtil.isNotEmpty(joinList)) {
			joinList.forEach(join -> {
				String childTable = join.getTarget();
				List<JoinKey> keyList = join.getJoinKeys();
				keyList.forEach(key -> {
					String childField = key.getTargetKey();
					RelationField parentRelation = RelationUtil.getParentRelationField(childTable,
							childField,
							joinList,
							fieldList);
					if (parentRelation != null) {
						// 该字段存在关联字段
						TableRowData childRow = tableRowMap.get(childTable);
						if (childRow != null) {
							TableFieldData childFieldData = childRow.getData(childField);
							// 取父表对应字段的值，并且赋值给该表字段
							TableRowData parentRow = tableRowMap.get(parentRelation.getTableName());
							String parentField = parentRelation.getField();
							String parentValue = parentRow.get(parentField).getValue();
							childFieldData.setValue(parentValue);
							childRow.add(childField, childFieldData);
						}
					}
				});
			});
		}

		// 每个表的行数据组装完成
		// 循环放入每个表的行数据
		tableKeys.forEach(table -> {
			TableRowData tableRow = tableRowMap.get(table);
			tableData.add(table, tableRow);
		});

		if (tableData.size() >= BATCH_COUNT) {
			saveData();
			tableData.clear();
		}
	}

	@Override
	public void doAfterAllAnalysed(AnalysisContext context) {
		saveData();
		log.info("所有数据解析完成！");
	}

	/**
	 * 将数据存储数据库
	 */
	private void saveData() {
		recursionInsert(relationTree);
		tableTotal += tableData.getTableSize();
		dataTotal += tableData.getDataSize();
		log.info("存储数据库成功！");
	}

	/**
	 * 递归从主表到次表，依次批量插入数据
	 * 
	 * @param tree 表关联关系的树形结构，从主表到次表
	 */
	private void recursionInsert(List<RelationBean> tree) {
		for (RelationBean relation : tree) {
			String tableName = relation.getTable();
			List<TableRowData> rowList = tableData.getRowList(tableName);
			if (CollUtil.isEmpty(rowList)) {
				continue;
			}

			TableRowData first = rowList.get(0);
			List<String> fieldSet = new ArrayList<>(first.keySet());

			if (CollUtil.isEmpty(fieldSet)) {
				continue;
			}

			// insert into t1(field1,field2) values(?,?);
			StringBuffer fieldBuffer = new StringBuffer();
			fieldBuffer.append("insert into " + tableName + "(");

			StringBuffer valueBuffer = new StringBuffer();
			valueBuffer.append(" values(");

			LinkedHashMap<String, ViewColumnTypeEnum> headerMap = new LinkedHashMap<>();

			for (int i = 0, size = fieldSet.size(); i < size; i++) {
				String field = fieldSet.get(i);
				fieldBuffer.append(field);
				valueBuffer.append("?");
				if (i < size - 1) {
					fieldBuffer.append(",");
					valueBuffer.append(",");
				}
				ViewColumnTypeEnum type = first.get(field).getType();
				headerMap.put(field, type);
			}

			fieldBuffer.append(")");
			valueBuffer.append(")");

			String insertSql = fieldBuffer.toString() + valueBuffer.toString();
			log.info("ExcelRead4MultipleTableListener::insert sql is " + insertSql);
			dynamicDataService.batchSaveForMultipleTable(ds, insertSql, BATCH_COUNT, headerMap, rowList);

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				// 插入子表数据
				recursionInsert(relation.getChildren());
			}
		}

		// log.info("insertSql is ->" + insertSql);
		// log.info("headerMap is ->" + JSON.toJSONString(header));
	}

	public long getDataTotal() {
		return dataTotal;
	}

	public long getTableTotal() {
		return tableTotal;
	}

	public String getKeyStart() {
		return keyStart;
	}

	public String getKeyEnd() {
		return keyEnd;
	}

}
