package org.springblade.db.excel.import_;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.db.cube.enums.ViewColumnTypeEnum;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.service.DataSourceService;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.database.DbBase;
import org.springblade.db.excel.import_.ExcelFormParam.ExcelFieldIndex;
import org.springblade.model.po.VisualdevEntity;
import org.springblade.service.ResetSerialnumberService;
import org.springblade.util.SystemComponent;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;

/**
 * 单表excel导入
 * 
 * @author william
 * @date 2022年9月15日 下午3:39:36
 */
@Slf4j
public class ExcelRead4SingleTableListener extends AnalysisEventListener<LinkedHashMap<Integer, String>> {

	private static final int BATCH_COUNT = 10000;

	private long total;

	private List<LinkedHashMap<Integer, String>> dataList = new ArrayList<>();

	private DataSourceModel ds;
	private List<ExcelFieldIndex> indexList;

	private String insertSql;
	private LinkedHashMap<Integer, ViewColumnTypeEnum> headerMap;
	private DynamicDataService dynamicDataService;
	private boolean hasKey = false;

	private boolean firstInvoke = true;
	private String keyStart;
	private String keyEnd;

	private String modelId;
	private Kv fieldOption;
	private Kv fieldDefault;
	private SystemComponent sysCompoment;

	public ExcelRead4SingleTableListener(DbBase dbBase,
			DataSourceService dataSourceService, CubeDataModel cubeData, DynamicDataService dynamicDataService,
			DataSourceModel ds, VisualdevEntity formInfo, List<ExcelFieldIndex> sourceIndexList, Kv fieldOption,
			Kv fieldDefault)
			throws SQLException {
		this.dynamicDataService = dynamicDataService;
		this.ds = ds;
		this.fieldOption = fieldOption;
		this.fieldDefault = fieldDefault;
		this.sysCompoment = SpringUtil.getBean(SystemComponent.class);
		this.modelId=formInfo.getId();
		headerMap = new LinkedHashMap<>();

		String tables = formInfo.getTables();
		List<RelationField> fieldList = JSON.parseArray(tables, RelationField.class);
		RelationField first = fieldList.get(0);

		String key = dbBase.getKey(ds, first.getTableName());

		// insert into t1(field1,field2) values(?,?);
		StringBuffer fieldBuffer = new StringBuffer();
		fieldBuffer.append("insert into " + first.getTableName() + "(");

		StringBuffer valueBuffer = new StringBuffer();
		valueBuffer.append(" values(");

		// 按照excel索引升序排列
		this.indexList = sourceIndexList.stream()
				.sorted(Comparator.comparing(ExcelFieldIndex::getIndex))
				.collect(Collectors.toList());
		for (int i = 0, size = indexList.size(); i < size; i++) {
			ExcelFieldIndex fieldIndex = indexList.get(i);
				RelationField field = getByFieldId(fieldList, fieldIndex.getFieldId());
				if (field != null){
					fieldBuffer.append(field.getField());
				}
				valueBuffer.append("?");
				if (i < size - 1) {
					fieldBuffer.append(",");
					valueBuffer.append(",");
				}
			if (fieldIndex.getIndex() !=-1){
				ViewColumnTypeEnum type = ViewColumnTypeEnum.valueOf(field.getDataTypeOrigin());
				headerMap.put(fieldIndex.getIndex(), type);

			}else {
				headerMap.put(fieldIndex.getIndex(), ViewColumnTypeEnum.STRING);
			}
		}

		if (StrUtil.isNotBlank(key)) {
			// 如果该表有主键
			hasKey = true;
			fieldBuffer.append("," + key);

			valueBuffer.append(",?");

			headerMap.put(9999, ViewColumnTypeEnum.BIGINT);
		}

		fieldBuffer.append(")");
		valueBuffer.append(")");

		insertSql = fieldBuffer.toString() + valueBuffer.toString();
	}

	private RelationField getByFieldId(List<RelationField> fieldList, int fieldId) {
		for (RelationField field : fieldList) {
			if (field.getFieldId() == fieldId) {
				return field;
			}
		}

		return null;
	}

	@Override
	public void invoke(LinkedHashMap<Integer, String> data, AnalysisContext context) {
		LinkedHashMap<Integer, String> temp = new LinkedHashMap<>();
		for (ExcelFieldIndex fieldIndex : indexList) {
			Integer index = fieldIndex.getIndex();
			if (index !=-1){
				String value = data.get(index);
				String fieldId = String.valueOf(fieldIndex.getFieldId());
				Kv fieldDefaultTemp = ((Kv) fieldDefault.get(fieldId));
				String jnpfKey = fieldDefaultTemp != null ? fieldDefaultTemp.getStr("jnpfKey") : "";
				if (sysCompoment.isSystemComponent(jnpfKey)) {
					// 系统默认组件，值由系统生成
					value = sysCompoment.getSystemValue(jnpfKey).toString();
				} else {

					boolean required=false;boolean primaryKey=false;boolean isJoined=false;
					String defValue = ((Kv) this.fieldDefault.get(fieldId)).getStr("defaultValue");
					if (this.fieldOption.containsKey("required")){
						 required = ((Kv) this.fieldOption.get(fieldId)).getBool("required");
					}
					if (this.fieldOption.containsKey("primaryKey")){
						 primaryKey = ((Kv) this.fieldOption.get(fieldId)).getBool("primaryKey");
					}
					if (this.fieldOption.containsKey("isJoined")){
						 isJoined = ((Kv) this.fieldOption.get(fieldId)).getBool("isJoined");
					}

					if ((required || isJoined) && !primaryKey && StrUtil.isBlank(defValue) && StrUtil.isBlank(value)) {
						// 必填，非主键，无默认值，并且要导入的数据为空,则跳过
						continue;
					}
				}

				temp.put(index, value);
			}else {
				List<Object> serialNumberConfig = fieldIndex.get_serialNumberConfig();
				String value=SpringUtil.getBean(ResetSerialnumberService.class).generateValue(modelId,serialNumberConfig,data,indexList);
				temp.put(index, value);
			}


		}
		if (hasKey) {
			String id = IdWorker.getIdStr();
			if (firstInvoke) {
				keyStart = id;
				firstInvoke = false;
			}
			keyEnd = id;
			temp.put(9999, id);
		}
		dataList.add(temp);

		if (dataList.size() >= BATCH_COUNT) {
			saveData();
			// 存储完成清理 list
			dataList.clear();
		}
	}

	@Override
	public void doAfterAllAnalysed(AnalysisContext context) {
		saveData();
		log.info("所有数据解析完成！");
	}

	/**
	 * 将数据存储数据库
	 */
	private void saveData() {
		log.info("insert sql is:" + insertSql);
		// log.info("dataList is:" + JSON.toJSONString(dataList));
		dynamicDataService.batchSave(ds, insertSql, BATCH_COUNT, headerMap, dataList);
		total += dataList.size();
		log.info("存储数据库成功！");
	}

	public long getTotal() {
		return total;
	}

	public String getKeyStart() {
		return keyStart;
	}

	public String getKeyEnd() {
		return keyEnd;
	}

}
