package org.springblade.db.sqlfly.config;

import java.io.Serializable;

import com.alibaba.fastjson2.JSON;

/**
 * 数据库的列
 * 
 * @author William
 * @date 2022年10月28日 上午11:26:41
 */
public class Column implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * 字段名
	 */
	private String columnName;

	/**
	 * 字段注释
	 */
	private String columnComment;
	
	/**
	 * 类型名称
	 */
	private String dataType;

	/**
	 * 是否可为空(0-否；1-是)
	 */
	private int isNullable;

	/**
	 * 是否为主键字段(0-否；1-是)
	 */
	private int primaryKeyValue;

	/**
	 * 长度
	 */
	private int size;

	/**
	 * 精度
	 */
	private int scale;

	/**
	 * 对应的java类型
	 */
	private String javaType;

	public String getColumnName() {
		return columnName;
	}

	public void setColumnName(String columnName) {
		this.columnName = columnName;
	}

	public String getDataType() {
		return dataType;
	}

	public void setDataType(String dataType) {
		this.dataType = dataType;
	}

	public String getColumnComment() {
		String realComment = columnComment;
		if (realComment.contains("[")) {
			String[] array = realComment.split("\\[");
			if (array != null && array.length > 0) {
				realComment = array[0];
			}
		}
		return realComment;
	}

	public void setColumnComment(String columnComment) {
		this.columnComment = columnComment;
	}

	public String getCommentExtra() {
		String commentExtra = null;
		if (columnComment.contains("[")) {
			String[] array = columnComment.split("\\[");
			if (array != null && array.length > 0) {
				commentExtra = array[1];
				if (commentExtra != null) {
					commentExtra = commentExtra.replace("]", "");
				}
			}
		}
		return commentExtra;
	}

	public void setIsNullable(int isNullable) {
		this.isNullable = isNullable;
	}

	public int getIsNullable() {
		return isNullable;
	}

	public int getPrimaryKeyValue() {
		return primaryKeyValue;
	}

	public void setPrimaryKeyValue(int primaryKeyValue) {
		this.primaryKeyValue = primaryKeyValue;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public int getScale() {
		return scale;
	}

	public void setScale(int scale) {
		this.scale = scale;
	}

	public String getJavaType() {
		return javaType;
	}

	public void setJavaType(String javaType) {
		this.javaType = javaType;
	}

	public boolean isPrimaryKey() {
		return primaryKeyValue == 1;
	}

	public boolean isRequired() {
		return isNullable == 0;
	}

	@Override
	public String toString() {
		return JSON.toJSONString(this);
	}

}
