package org.springblade.db.sqlfly.opt;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.config.Table;
import org.springblade.db.sqlfly.type.ClickhouseType;
import org.springblade.db.sqlfly.type.JavaType;

import com.alibaba.fastjson2.JSON;

import cn.hutool.json.JSONUtil;

/**
 * clickhouse数据库操作
 * 
 * @author William
 * @date 2022年10月28日 上午11:29:37
 */
public class FlyClickhouse extends IFly {

	protected FlyClickhouse() {
	}

	@Override
	public List<Table> getTableList(String schema) {
		List<Table> tables = new ArrayList<>();
		String sql = "select table  from system.parts where database = '" + db.getDbName() + "' group by table";
		List<String> retList = fly.getList(String.class, sql);
		retList.forEach(item -> tables.add(new Table(item)));
		return tables;
	}

	@Override
	public Table getTable(String schema, String tableName) {
		return new Table(tableName);
	}

	@Override
	public List<Column> getColumnList(String tableName) {
		List<Column> list = new ArrayList<>();
		String sql = "SELECT\r\n"
				+ "	c.name as column_name,\r\n"
				+ "	c.comment as column_comment,\r\n"
				+ "	c.type as data_type,\r\n"
				+ "	c.is_in_primary_key as primary_key_value\r\n"
				+ "FROM\r\n"
				+ "	system.columns c\r\n"
				+ "WHERE\r\n"
				+ "	database = '" + db.getDbName() + "'\r\n"
				+ "	AND table = '" + tableName + "'";
		fly.setConnection(null);
		List<Map<String, Object>> retList = fly.getMapList(sql);
		String json = JSON.toJSONString(retList);
		List<Column> columns = JSONUtil.toList(json, Column.class);

		JavaType javaType = new ClickhouseType();
		columns.forEach(col -> {
			col.setJavaType(javaType.get(col.getDataType()));
			String dataType = col.getDataType();
			if (dataType.contains("Nullable")) {
				col.setIsNullable(1);
			} else {
				col.setIsNullable(0);
			}
			dataType = dataType.replace("Nullable", "").replace("(", "").replace(")", "");
			col.setDataType(dataType);
		});
		return list;
	}

}
