package org.springblade.db.sqlfly.opt;

import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.config.Table;
import org.springblade.db.sqlfly.type.JavaType;
import org.springblade.db.sqlfly.type.MysqlType;

import com.alibaba.fastjson2.JSON;

import cn.hutool.json.JSONUtil;

/**
 * mysql数据库操作
 * 
 * @author William
 * @date 2022年10月28日 上午11:29:54
 */

@Slf4j
public class FlyMysql extends IFly {

	protected FlyMysql() {
	}

	@Override
	public List<Table> getTableList(String schema) {
		String sql = "select table_name, table_comment, create_time, update_time from information_schema.tables where table_schema = (select database())";
		List<Map<String, Object>> retList = fly.getMapList(sql);
		String json = JSON.toJSONString(retList);
		List<Table> list = JSONUtil.toList(json, Table.class);
		return list;
	}

	public Table getTable(String schema, String tableName) {
		String sql = "select table_name, table_comment, create_time, update_time from information_schema.tables where table_schema = (select database()) and table_name = '"
				+ tableName + "'";
		Map<String, Object> ret = fly.getMap(sql);
		String json = JSON.toJSONString(ret);
		Table table = JSONUtil.toBean(json, Table.class);
		return table;
	}

	@Override
	public List<Column> getColumnList(String tableName) {
		String sql = "SELECT \r\n"
				+ "       column_comment,\r\n"
				+ "       column_name,\r\n"
				+ "       data_type,\r\n"
				+ "       (\r\n"
				+ "           CASE\r\n"
				+ "               WHEN data_type = 'float'\r\n"
				+ "                   OR data_type = 'double'\r\n"
				+ "                   OR data_type = 'TINYINT'\r\n"
				+ "                   OR data_type = 'SMALLINT'\r\n"
				+ "                   OR data_type = 'MEDIUMINT'\r\n"
				+ "                   OR data_type = 'INT'\r\n"
				+ "                   OR data_type = 'INTEGER'\r\n"
				+ "                   OR data_type = 'decimal'\r\n"
				+ "                   OR data_type = 'bigint'\r\n"
				+ "                   THEN\r\n"
				+ "                   NUMERIC_PRECISION\r\n"
				+ "               ELSE\r\n"
				+ "                   CHARACTER_MAXIMUM_LENGTH\r\n"
				+ "               END\r\n"
				+ "           )            AS 'size',\r\n"
				+ "       NUMERIC_SCALE    AS 'scale',\r\n"
				+ "       (\r\n"
				+ "           CASE\r\n"
				+ "               WHEN EXTRA = 'auto_increment' THEN\r\n"
				+ "                   1\r\n"
				+ "               ELSE\r\n"
				+ "                   0\r\n"
				+ "               END\r\n"
				+ "           )            AS 'auto_increment',\r\n"
				+ "              (\r\n"
				+ "           CASE\r\n"
				+ "               WHEN IS_NULLABLE = 'NO' THEN\r\n"
				+ "                   0\r\n"
				+ "               ELSE\r\n"
				+ "                   1\r\n"
				+ "               END\r\n"
				+ "           )            AS 'is_nullable',\r\n"
				+ "       (\r\n"
				+ "           CASE\r\n"
				+ "               WHEN COLUMN_KEY = 'PRI' THEN\r\n"
				+ "                   1\r\n"
				+ "               ELSE\r\n"
				+ "                   0\r\n"
				+ "               END\r\n"
				+ "           )            AS 'primary_key_value',\r\n"
				+ " \r\n"
//				+ "        #extra,\r\n"
//				+ "        #is_nullable,\r\n"
				+ "       column_default\r\n"
				+ "from\r\n"
				+ "information_schema.COLUMNS\r\n"
				+ "where\r\n"
				+ "table_name = '" + tableName + "'\r\n"
				+ "and table_schema = (\r\n"
				+ "select\r\n"
				+ "		database())";

		fly.setConnection(null);
		List<Map<String, Object>> retList = fly.getMapList(sql);
		String json = JSON.toJSONString(retList);
		List<Column> columns = JSONUtil.toList(json, Column.class);
		JavaType javaType = new MysqlType();
		columns.forEach(col -> {
			col.setJavaType(javaType.get(col.getDataType()));
		});
		return columns;
	}
}
