package org.springblade.service;

import java.io.File;
import java.math.BigDecimal;
import java.util.Date;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.db.database.FormDataService;
import org.springblade.db.excel.import_.ExcelFormParam;
import org.springblade.mapper.DataImportLogMapper;
import org.springblade.model.po.DataImportLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 
 * @author William
 * @date 2022年11月17日 上午11:55:38
 */
@Service
public class DataImportLogService extends ServiceImpl<DataImportLogMapper, DataImportLog> {
	@Value("${blade.file.uploadPath}")
	private String uploadDir;

	@Autowired
	private FormDataService formDataService;

	/**
	 * 保存导入日志
	 * 
	 * @param params
	 * @param count
	 * @param cubeId
	 */
	public void saveLog(ExcelFormParam params, long count, Integer cubeId, float takeUpTime, String keyStart,
			String keyEnd) {
		long fileSize = 0;
		String filePath = uploadDir + params.getFile().replace("uploads/", "");
		File excelFile = new File(filePath);
		if (excelFile != null && excelFile.exists()) {
			fileSize = excelFile.length();
		}

		DataImportLog bean = DataImportLog.builder()
				.count(count)
				.createTime(new Date())
				.cubeId(cubeId)
				.filePath(params.getFile())
				.fileSize(fileSize)
				.finishTime(new Date())
				.isTruncate(params.isTruncate() ? 1 : 0)
				.modelId(params.getModelId())
				.name(params.getName())
				.operatorId(SecureUtil.getUserId())
				.operator(SecureUtil.getNickName())
				.rule(JSON.toJSONString(params.getList()))
				.status(1)
				.takeUpTime(BigDecimal.valueOf(takeUpTime))
				.withHeader(params.getWithHeader() ? 1 : 0)
				.keyStart(keyStart)
				.keyEnd(keyEnd)
				.build();
		save(bean);
	}

	/**
	 * 撤销导入
	 * 
	 * @param dataLog
	 * @param keyStart
	 * @param keyEnd
	 * @param hasLinkTable
	 */
	public R<String> revoke(DataImportLog dataLog, String keyStart, String keyEnd, boolean hasLinkTable) {
		R<String> ret = formDataService.delete(dataLog.getModelId(), keyStart, keyEnd, hasLinkTable);
		if (ret.isSuccess()) {
			// 撤销成功后，删除导入日志
			removeById(dataLog.getId());
		}
		return ret;
	}
}
