package org.springblade.service;

import java.util.List;

import org.springblade.mapper.DataInterfaceMapper;
import org.springblade.model.dto.PaginationDataInterface;
import org.springblade.model.po.DataInterfaceEntity;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.util.StrUtil;

/**
 * 
 * @author William
 * @date 2022年12月30日 下午2:07:07
 */
@Service
public class DataInterfaceService extends ServiceImpl<DataInterfaceMapper, DataInterfaceEntity>{

	public List<DataInterfaceEntity> getList(PaginationDataInterface pagination) {
		QueryWrapper<DataInterfaceEntity> queryWrapper = new QueryWrapper<>();
		// 关键字
		if (!StrUtil.isEmpty(pagination.getKeyword())) {
			queryWrapper.lambda().and(t -> t.like(DataInterfaceEntity::getFullName, pagination.getKeyword()).or()
					.like(DataInterfaceEntity::getEnCode, pagination.getKeyword()));
		}
		// 分类
		queryWrapper.lambda().eq(DataInterfaceEntity::getCategoryId, pagination.getCategoryId());
		// 排序
		queryWrapper.lambda().orderByAsc(DataInterfaceEntity::getSortCode);
		Page<DataInterfaceEntity> page = new Page<>(pagination.getCurrentPage(), pagination.getPageSize());
		IPage<DataInterfaceEntity> iPage = this.page(page, queryWrapper);
		return pagination.setData(iPage.getRecords(), iPage.getTotal());
	}
	
	public List<DataInterfaceEntity> getList() {
		QueryWrapper<DataInterfaceEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DataInterfaceEntity::getEnabledMark, "1");
		return baseMapper.selectList(queryWrapper);
	}
	
	public boolean isExistByFullName(String fullName, String id) {
		QueryWrapper<DataInterfaceEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DataInterfaceEntity::getFullName, fullName);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DataInterfaceEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}
	
	public boolean isExistByEnCode(String enCode, String id) {
		QueryWrapper<DataInterfaceEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DataInterfaceEntity::getEnCode, enCode);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DataInterfaceEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}
	
}
