package org.springblade.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.database.FormDataService;
import org.springblade.mapper.DataRecycleMapper;
import org.springblade.model.po.DataRecycle;
import org.springblade.model.po.VisualdevEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 
 * @author William
 * @date 2022年12月26日 上午10:53:32
 */
@Slf4j
@Service
public class DataRecycleService extends ServiceImpl<DataRecycleMapper, DataRecycle> {

	@Autowired
	private FormDataService formDataService;

	@Autowired
	private VisualdevService visualdevService;

	public boolean save(String modelId, LinkedHashMap<String, List<JSONObject>> data) {
		List<Kv> dataViewList = new ArrayList<>();
		VisualdevEntity formInfo = visualdevService.getById(modelId);
		if (formInfo != null) {
			List<RelationField> fields = JSON.parseArray(formInfo.getTables(), RelationField.class);
			fields.forEach(rf -> {
				Kv dv = Kv.create();
				String column = rf.getField();
				String label = rf.getFieldName();
				String table = rf.getTableName();
				int fieldId = rf.getFieldId();
				dv.set("label", label).set("column", column).set("table", table).set("fieldId",fieldId);
				List<JSONObject> dataOfTable = data.get(table);
				if (dataOfTable != null){
					dataOfTable.forEach(dataOfRow -> {
						String value = dataOfRow.getString(column);
						dv.set("value", value);
					});
					dataViewList.add(dv);
				}

			});
		}
		String dataView = JSON.toJSONString(dataViewList);

		DataRecycle bean = DataRecycle.builder()
				.createTime(new Date())
				.data(JSON.toJSONString(data))
				.dataView(dataView)
				.modelId(modelId)
				.operator(SecureUtil.getNickName())
				.operatorId(SecureUtil.getUserId())
				.build();
		return save(bean);
	}

	public boolean recovery(List<String> ids) {
		List<DataRecycle> dataList = listByIds(ids);
		if (CollUtil.isEmpty(dataList)) {
			return false;
		}
		try {
			for (DataRecycle bean : dataList) {
				String modelId = bean.getModelId();
				String dataJson = bean.getData();
				JSONObject data = JSON.parseObject(dataJson);
				if (data != null) {
					Set<String> keySet = data.keySet();
					if (CollUtil.isNotEmpty(keySet)) {
						for (String table : keySet) {
							JSONArray list = data.getJSONArray(table);
							if (CollUtil.isNotEmpty(list)) {
								formDataService.recovery(formDataService.getDs(modelId), table, list);
							}
						}
					}
				}
			}
			// 从回收站删除已经恢复的数据
			removeByIds(ids);
		} catch (Exception e) {
			log.info("recovery exception is " + e.toString());
			return false;
		}
		return true;
	}

}
