package org.springblade.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.db.exception.DataException;
import org.springblade.mapper.DictionaryDataMapper;
import org.springblade.model.dto.DictDataExportModel;
import org.springblade.model.dto.DictionaryExportModel;
import org.springblade.model.po.DictionaryDataEntity;
import org.springblade.model.po.DictionaryTypeEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.pinyin.PinyinUtil;

/**
 * 
 * @author William
 * @date 2022年12月13日 下午6:52:16
 */
@Service
public class DictionaryDataService extends ServiceImpl<DictionaryDataMapper, DictionaryDataEntity> {

	@Autowired
	private DictionaryTypeService dictTypeService;

	public List<DictionaryDataEntity> getListForVisualdev() {
		DictionaryTypeEntity type = dictTypeService.getInfoByEnCode("webDesign");
		if (type == null || type.getId() == null) {
			return null;
		}
		LambdaQueryWrapper<DictionaryDataEntity> wrapper = Wrappers.<DictionaryDataEntity>lambdaQuery();
		wrapper.eq(DictionaryDataEntity::getParentId, type.getId());
		wrapper.eq(DictionaryDataEntity::getModeType, 2);
		wrapper.orderByDesc(DictionaryDataEntity::getSortCode);
		return list(wrapper);
	}
	

	public List<DictionaryDataEntity> getList() {
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().orderByAsc(DictionaryDataEntity::getSortCode);
		return this.list(queryWrapper);
	}

	public List<DictionaryDataEntity> getList(String dictTypeId) {
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.and(
						t -> t.eq(DictionaryDataEntity::getDictionaryTypeId, dictTypeId)
								.or()
								.eq(DictionaryDataEntity::getEnCode, dictTypeId));
		queryWrapper.lambda().orderByAsc(DictionaryDataEntity::getSortCode);
		return this.list(queryWrapper);
	}

	public DictionaryDataEntity getInfo(String id) {
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DictionaryDataEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	public boolean isExistByFullName(String dictTypeId, String fullName, String id) {
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.eq(DictionaryDataEntity::getFullName, fullName)
				.eq(DictionaryDataEntity::getDictionaryTypeId, dictTypeId);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DictionaryDataEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}

	public boolean isExistByEnCode(String dictionaryTypeId, String enCode, String id) {
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.eq(DictionaryDataEntity::getEnCode, enCode)
				.eq(DictionaryDataEntity::getDictionaryTypeId, dictionaryTypeId);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DictionaryDataEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}

	public void delete(DictionaryDataEntity entity) {
		this.removeById(entity.getId());
	}

	public void create(DictionaryDataEntity entity) {
		// 判断id是否为空,为空则为新建
		if (StrUtil.isEmpty(entity.getId())) {
			entity.setId(IdUtil.fastSimpleUUID());
			entity.setSimpleSpelling(PinyinUtil.getFirstLetter(entity.getFullName(), "").toUpperCase());
			entity.setCreatorUserId(String.valueOf(SecureUtil.getUserId()));
		}
		this.save(entity);
	}

	public boolean update(String id, DictionaryDataEntity entity) {
		entity.setId(id);
		entity.setLastModifyTime(new Date());
		entity.setLastModifyUserId(String.valueOf(SecureUtil.getUserId()));
		return this.updateById(entity);
	}

	public boolean first(String id) {
		boolean isOk = false;
		// 获取要上移的那条数据的信息
		DictionaryDataEntity upEntity = this.getById(id);
		Long upSortCode = upEntity.getSortCode() == null ? 0 : upEntity.getSortCode();
		// 查询上几条记录
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.eq(DictionaryDataEntity::getDictionaryTypeId, upEntity.getDictionaryTypeId())
				.eq(DictionaryDataEntity::getParentId, upEntity.getParentId())
				.lt(DictionaryDataEntity::getSortCode, upSortCode)
				.orderByDesc(DictionaryDataEntity::getSortCode);
		List<DictionaryDataEntity> downEntity = this.list(queryWrapper);
		if (downEntity.size() > 0) {
			// 交换两条记录的sort值
			Long temp = upEntity.getSortCode();
			upEntity.setSortCode(downEntity.get(0).getSortCode());
			downEntity.get(0).setSortCode(temp);
			updateById(downEntity.get(0));
			updateById(upEntity);
			isOk = true;
		}
		return isOk;
	}

	public boolean next(String id) {
		boolean isOk = false;
		// 获取要下移的那条数据的信息
		DictionaryDataEntity downEntity = this.getById(id);
		Long upSortCode = downEntity.getSortCode() == null ? 0 : downEntity.getSortCode();
		// 查询下几条记录
		QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.eq(DictionaryDataEntity::getDictionaryTypeId, downEntity.getDictionaryTypeId())
				.eq(DictionaryDataEntity::getParentId, downEntity.getParentId())
				.gt(DictionaryDataEntity::getSortCode, upSortCode)
				.orderByAsc(DictionaryDataEntity::getSortCode);
		List<DictionaryDataEntity> upEntity = this.list(queryWrapper);
		if (upEntity.size() > 0) {
			// 交换两条记录的sort值
			Long temp = downEntity.getSortCode();
			downEntity.setSortCode(upEntity.get(0).getSortCode());
			upEntity.get(0).setSortCode(temp);
			updateById(upEntity.get(0));
			updateById(downEntity);
			isOk = true;
		}
		return isOk;
	}

	public List<DictionaryDataEntity> getDictionName(List<String> id) {
		List<DictionaryDataEntity> dictionList = new ArrayList<>();
		if (id.size() > 0) {
			QueryWrapper<DictionaryDataEntity> queryWrapper = new QueryWrapper<>();
			queryWrapper.lambda().in(DictionaryDataEntity::getEnCode, id);
			dictionList = this.list(queryWrapper);
		}
		return dictionList;
	}

	public DictionaryExportModel exportData(String id) {
		// 获取数据分类字段详情
		DictionaryTypeEntity typeEntity = dictTypeService.getInfo(id);
		DictionaryExportModel exportModel = new DictionaryExportModel();
		// 递归子分类
		Set<DictionaryTypeEntity> set = new HashSet<>();
		List<DictionaryTypeEntity> typeEntityList = dictTypeService.getList();
		getDictionaryTypeEntitySet(typeEntity, set, typeEntityList);
		List<DictionaryTypeEntity> collect = new ArrayList<>(set);
		// 判断是否有子分类
		if (collect.size() > 0) {
			exportModel.setList(collect);
		}
		// 判断是否需要new
		if (exportModel.getList() == null) {
			List<DictionaryTypeEntity> list = new ArrayList<>();
			list.add(typeEntity);
			exportModel.setList(list);
		} else {
			exportModel.getList().add(typeEntity);
		}
		// 获取该类型下的数据
		List<DictDataExportModel> modelList = new ArrayList<>();
		for (DictionaryTypeEntity dictionaryTypeEntity : exportModel.getList()) {
			List<DictionaryDataEntity> entityList = getList(dictionaryTypeEntity.getId());
			for (DictionaryDataEntity dde : entityList) {
				DictDataExportModel dem = JSON.parseObject(JSON.toJSONString(dde), DictDataExportModel.class);
				modelList.add(dem);
			}
		}
		exportModel.setModelList(modelList);
		return exportModel;
	}

	/**
	 * 递归字典分类
	 * 
	 * @param dictionaryTypeEntity 数据字典类型实体
	 */
	private void getDictionaryTypeEntitySet(DictionaryTypeEntity dictionaryTypeEntity, Set<DictionaryTypeEntity> set,
			List<DictionaryTypeEntity> typeEntityList) {
		// 是否含有子分类
		List<DictionaryTypeEntity> collect = typeEntityList.stream()
				.filter(t -> dictionaryTypeEntity.getId().equals(t.getParentId()))
				.collect(Collectors.toList());
		if (collect.size() > 0) {
			for (DictionaryTypeEntity typeEntity : collect) {
				set.add(typeEntity);
				getDictionaryTypeEntitySet(typeEntity, set, typeEntityList);
			}
		}
	}

	@Transactional(rollbackFor = DataException.class)
	public boolean importData(DictionaryExportModel exportModel) throws DataException {
		try {
			boolean exist = true;
			List<DictionaryTypeEntity> list = JSON.parseArray(JSON.toJSONString(exportModel.getList()),
					DictionaryTypeEntity.class);
			List<DictionaryDataEntity> entityList = JSON.parseArray(JSON.toJSONString(exportModel.getModelList()),
					DictionaryDataEntity.class);
			// 遍历插入分类
			for (DictionaryTypeEntity dict : list) {
				if (dictTypeService.getInfo(dict.getId()) == null
						&& !dictTypeService.isExistByEnCode(dict.getEnCode(), dict.getId())
						&& !dictTypeService.isExistByFullName(dict.getFullName(), dict.getId())) {
					exist = false;
					dictTypeService.create(dict);
				}
			}
			for (DictionaryDataEntity dict : entityList) {
				DictionaryDataEntity dataEntity = getInfo(dict.getId());
				if (dataEntity == null && dictTypeService.getInfo(dict.getDictionaryTypeId()) != null
						&& !isExistByFullName(dict.getDictionaryTypeId(), dict.getFullName(), dict.getId())
						&& !isExistByEnCode(dict.getDictionaryTypeId(), dict.getEnCode(), dict.getId())) {
					exist = false;
					create(dict);
				}
			}
			return exist;
		} catch (Exception e) {
			// 手动回滚事务
			TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
			throw new DataException(e.getMessage());
		}
	}

}
