package org.springblade.service;

import java.util.Date;
import java.util.List;

import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.mapper.DictionaryTypeMapper;
import org.springblade.model.po.DictionaryTypeEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 
 * @author William
 * @date 2022年12月13日 下午7:05:01
 */
@Service
public class DictionaryTypeService extends ServiceImpl<DictionaryTypeMapper, DictionaryTypeEntity> {

	public List<DictionaryTypeEntity> getList() {
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.orderByDesc(DictionaryTypeEntity::getSortCode)
				.orderByDesc(DictionaryTypeEntity::getCreatorTime);
		return this.list(queryWrapper);
	}

	public DictionaryTypeEntity getInfoByEnCode(String enCode) {
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DictionaryTypeEntity::getEnCode, enCode);
		return this.getOne(queryWrapper);
	}

	public DictionaryTypeEntity getInfo(String id) {
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DictionaryTypeEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	public boolean isExistByFullName(String fullName, String id) {
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DictionaryTypeEntity::getFullName, fullName);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DictionaryTypeEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}

	public boolean isExistByEnCode(String enCode, String id) {
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(DictionaryTypeEntity::getEnCode, enCode);
		if (!StrUtil.isEmpty(id)) {
			queryWrapper.lambda().ne(DictionaryTypeEntity::getId, id);
		}
		return this.count(queryWrapper) > 0 ? true : false;
	}

	public void create(DictionaryTypeEntity entity) {
		// 判断id是否为空,为空则为新建
		if (StrUtil.isEmpty(entity.getId())) {
			entity.setId(IdUtil.fastSimpleUUID());
			entity.setCreatorUserId(String.valueOf(SecureUtil.getUserId()));
		}
		this.save(entity);
	}

	public boolean update(String id, DictionaryTypeEntity entity) {
		entity.setId(id);
		entity.setLastModifyTime(new Date());
		entity.setLastModifyUserId(String.valueOf(SecureUtil.getUserId()));
		return this.updateById(entity);
	}

	public void delete(DictionaryTypeEntity entity) {
		this.removeById(entity.getId());
	}

	@Transactional
	public boolean first(String id) {
		boolean isOk = false;
		// 获取要上移的那条数据的信息
		DictionaryTypeEntity upEntity = this.getById(id);
		Long upSortCode = upEntity.getSortCode() == null ? 0 : upEntity.getSortCode();
		// 查询上几条记录
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.lt(DictionaryTypeEntity::getSortCode, upSortCode)
				.eq(DictionaryTypeEntity::getParentId, upEntity.getParentId())
				.orderByDesc(DictionaryTypeEntity::getSortCode);
		List<DictionaryTypeEntity> downEntity = this.list(queryWrapper);
		if (downEntity.size() > 0) {
			// 交换两条记录的sort值
			Long temp = upEntity.getSortCode();
			upEntity.setSortCode(downEntity.get(0).getSortCode());
			downEntity.get(0).setSortCode(temp);
			this.updateById(downEntity.get(0));
			this.updateById(upEntity);
			isOk = true;
		}
		return isOk;
	}

	@Transactional
	public boolean next(String id) {
		boolean isOk = false;
		// 获取要下移的那条数据的信息
		DictionaryTypeEntity downEntity = this.getById(id);
		Long upSortCode = downEntity.getSortCode() == null ? 0 : downEntity.getSortCode();
		// 查询下几条记录
		QueryWrapper<DictionaryTypeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.gt(DictionaryTypeEntity::getSortCode, upSortCode)
				.eq(DictionaryTypeEntity::getParentId, downEntity.getParentId())
				.orderByAsc(DictionaryTypeEntity::getSortCode);
		List<DictionaryTypeEntity> upEntity = this.list(queryWrapper);
		if (upEntity.size() > 0) {
			// 交换两条记录的sort值
			Long temp = downEntity.getSortCode();
			downEntity.setSortCode(upEntity.get(0).getSortCode());
			upEntity.get(0).setSortCode(temp);
			updateById(upEntity.get(0));
			updateById(downEntity);
			isOk = true;
		}
		return isOk;
	}

}
