package org.springblade.service;

import java.util.List;

import org.springblade.mapper.OrganizeMapper;
import org.springblade.model.po.OrganizeEntity;
import org.springblade.model.po.UserEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollUtil;

/**
 *  服务类
 *
 * @author william
 * @date   2022-07-14 10:57:52
 */
@Service
public class OrganizeService extends ServiceImpl<OrganizeMapper, OrganizeEntity> {

	@Autowired
	private UserService userService;

	public List<OrganizeEntity> getList() {
		QueryWrapper<OrganizeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().orderByAsc(OrganizeEntity::getSortCode);
		List<OrganizeEntity> list = this.list(queryWrapper);
		return list;
	}

	public OrganizeEntity getInfo(String id) {
		QueryWrapper<OrganizeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(OrganizeEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	public List<OrganizeEntity> getOrgRedisList() {
		QueryWrapper<OrganizeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(OrganizeEntity::getIsDeleted, 0);
		List<OrganizeEntity> list = this.list(queryWrapper);
		return list;
	}

	/**
	 * 是否有子部门
	 * 
	 * @return
	 */
	public boolean hasChild(String id) {
		QueryWrapper<OrganizeEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(OrganizeEntity::getParentId, id);
		List<OrganizeEntity> list = this.list(queryWrapper);
		return CollUtil.isNotEmpty(list);
	}

	/**
	 * 是否有员工
	 * 
	 * @return
	 */
	public boolean hasUser(String id) {
		QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(UserEntity::getOrganizeId, id);
		List<UserEntity> list = userService.list(queryWrapper);
		return CollUtil.isNotEmpty(list);
	}
}