package org.springblade.service;

import java.util.ArrayList;
import java.util.List;

import org.springblade.mapper.RoleMapper;
import org.springblade.model.po.RoleEntity;
import org.springblade.model.vo.RoleSelectorVO;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 *  服务类
 *
 * @author william
 * @date   2022-07-14 10:57:53
 */
@Service
public class RoleService extends ServiceImpl<RoleMapper, RoleEntity> {

	public List<RoleEntity> getList(String keyword) {
		QueryWrapper<RoleEntity> queryWrapper = new QueryWrapper<>();
		if (keyword != null) {
			queryWrapper.lambda().and(t -> t.like(RoleEntity::getFullName, keyword).or()
					.like(RoleEntity::getRoleAlias, keyword));
		}
		queryWrapper.lambda().orderByAsc(RoleEntity::getSort);
		return this.list(queryWrapper);
	}

	public RoleEntity getInfo(String id) {
		QueryWrapper<RoleEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(RoleEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	public List<RoleEntity> getRoleName(List<String> id) {
		List<RoleEntity> roleList = new ArrayList<>();
		if (id.size() > 0) {
			QueryWrapper<RoleEntity> queryWrapper = new QueryWrapper<>();
			queryWrapper.lambda().in(RoleEntity::getId, id);
			roleList = this.list(queryWrapper);
		}
		return roleList;
	}

	public List<RoleEntity> getList() {
		return this.list();
	}

	public List<RoleSelectorVO> getTreeList() {
		QueryWrapper<RoleEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().orderByAsc(RoleEntity::getId);
		List<RoleEntity> list = this.list(queryWrapper);
		List<RoleSelectorVO> treeList = JSON.parseArray(JSON.toJSONString(list), RoleSelectorVO.class);
		treeList = buildTree(treeList);
		return treeList;
	}

	private List<RoleSelectorVO> buildTree(List<RoleSelectorVO> list) {
		List<RoleSelectorVO> treeList = new ArrayList<>();
		for (RoleSelectorVO item : list) {
			if ("0".equalsIgnoreCase(item.getParentId())) {
				treeList.add(this.buildChildren(item, list));
			}
		}
		return treeList;
	}

	private RoleSelectorVO buildChildren(RoleSelectorVO parent, List<RoleSelectorVO> list) {
		for (RoleSelectorVO item : list) {
			if (item.getParentId().equalsIgnoreCase(parent.getId())) {
				if (parent.getChildren() == null) {
					parent.setChildren(new ArrayList<>());
				}
				parent.getChildren().add(this.buildChildren(item, list));
			}
		}
		return parent;
	}
}