package org.springblade.util;

import org.springblade.config.JnpfKeyConsts;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.model.dto.SysUser;
import org.springblade.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.github.xiaoymin.knife4j.core.util.StrUtil;

/**
 * 
 * @author William
 * @date 2023年2月3日 下午12:00:53
 */
@Component
public class SystemComponent {

	@Autowired
	private UserService userService;

	public boolean isSystemComponent(String jnpfKey) {
		if (StrUtil.isBlank(jnpfKey)) {
			return false;
		}

		for (String key : JnpfKeyConsts.SYSTEM_COMPONENT) {
			if (jnpfKey.equalsIgnoreCase(key)) {
				return true;
			}
		}
		return false;
	}

	public Object getSystemValue(String jnpfKey) {
		SysUser user = userService.get(SecureUtil.getUserId());
		switch (jnpfKey) {
		case JnpfKeyConsts.CREATEUSERID:
		case JnpfKeyConsts.MODIFYUSERID:
			return String.valueOf(SecureUtil.getUserId());
		case JnpfKeyConsts.CREATEUSER:
		case JnpfKeyConsts.MODIFYUSER:
			return user.getName();
		case JnpfKeyConsts.CREATETIME:
		case JnpfKeyConsts.MODIFYTIME:
			return System.currentTimeMillis();
		case JnpfKeyConsts.CURRORGANIZE:
			return user.getDept();
		case JnpfKeyConsts.CURRDEPT:
			return user.getDept();
		case JnpfKeyConsts.CURRPOSITION:
			return user.getPost();
		}
		return "";
	}
}
